@@ -1,6 +1,11 @@
 /*
- * Copyright (c) 1992, Brian Berliner and Jeff Polk
- * Copyright (c) 1989-1992, Brian Berliner
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
+ *
+ * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
+ *                                  and others.
+ *
+ * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
+ * Portions Copyright (C) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -33,8 +38,20 @@
  * edited by the user, if necessary (when the repository is moved, e.g.)
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* CVS headers.  */
+#include "ignore.h"
+#include "recurse.h"
+#include "repos.h"
+#include "wrapper.h"
+
 #include "cvs.h"
 
+
+
 static char *findslash (char *start, char *p);
 static int checkout_proc (int argc, char **argv, char *where,
 		          char *mwhere, char *mfile, int shorten,
@@ -66,13 +83,13 @@ static const char *const checkout_usage[] =
 
 static const char *const export_usage[] =
 {
-    "Usage: %s %s [-NRfln] [-r rev] [-D date] [-d dir] [-k kopt] module...\n",
+    "Usage: %s %s [-NRfln] [-r tag] [-D date] [-d dir] [-k kopt] module...\n",
     "\t-N\tDon't shorten module paths if -d specified.\n",
     "\t-f\tForce a head revision match if tag/date not found.\n",
     "\t-l\tLocal directory only, not recursive\n",
     "\t-R\tProcess directories recursively (default).\n",
     "\t-n\tDo not run module program (if any).\n",
-    "\t-r rev\tExport revision or tag.\n",
+    "\t-r tag\tExport tagged revisions.\n",
     "\t-D date\tExport revisions as of date.\n",
     "\t-d dir\tExport into dir instead of module name.\n",
     "\t-k kopt\tUse RCS kopt -k option on checkout.\n",
@@ -86,11 +103,11 @@ static int pipeout;
 static int aflag;
 static char *options;
 static char *tag;
-static int tag_validated;
+static bool tag_validated;
 static char *date;
-static char *join_rev1;
-static char *join_rev2;
-static int join_tags_validated;
+static char *join_rev1, *join_date1;
+static char *join_rev2, *join_date2;
+static bool join_tags_validated;
 static char *preload_update_dir;
 static char *history_name;
 static enum mtype m_type;
@@ -108,6 +125,7 @@ checkout (int argc, char **argv)
     char *where = NULL;
     const char *valid_options;
     const char *const *valid_usage;
+    char *join_orig1, *join_orig2;
 
     /* initialize static options */
     force_tag_match = 1;
@@ -116,9 +134,10 @@ checkout (int argc, char **argv)
 	free (options);
 	options = NULL;
     }
-    tag = date = join_rev1 = join_rev2 = preload_update_dir = NULL;
+    tag = date = join_rev1 = join_date1 = join_rev2 = join_date2 =
+	  join_orig1 = join_orig2 = preload_update_dir = NULL;
     history_name = NULL;
-    tag_validated = join_tags_validated = 0;
+    tag_validated = join_tags_validated = false;
 
 
     /*
@@ -167,11 +186,9 @@ checkout (int argc, char **argv)
 		break;
 	    case 'Q':
 	    case 'q':
-#ifdef SERVER_SUPPORT
 		/* The CVS 1.5 client sends these options (in addition to
 		   Global_option requests), so we must ignore them.  */
 		if (!server_active)
-#endif
 		    error (1, 0,
 			   "-q or -Q must be specified before \"%s\"",
 			   cvs_cmd_name);
@@ -205,20 +222,29 @@ checkout (int argc, char **argv)
 		force_tag_match = 0;
 		break;
 	    case 'r':
-		tag = optarg;
+		parse_tagdate (&tag, &date, optarg);
 		checkout_prune_dirs = 1;
 		break;
 	    case 'D':
+		if (date) free (date);
 		date = Make_Date (optarg);
 		checkout_prune_dirs = 1;
 		break;
 	    case 'j':
-		if (join_rev2)
+		if (join_rev2 || join_date2)
 		    error (1, 0, "only two -j options can be specified");
-		if (join_rev1)
-		    join_rev2 = optarg;
+		if (join_rev1 || join_date1)
+		{
+		    if (join_orig2) free (join_orig2);
+		    join_orig2 = xstrdup (optarg);
+		    parse_tagdate (&join_rev2, &join_date2, optarg);
+		}
 		else
-		    join_rev1 = optarg;
+		{
+		    if (join_orig1) free (join_orig1);
+		    join_orig1 = xstrdup (optarg);
+		    parse_tagdate (&join_rev1, &join_date1, optarg);
+		}
 		break;
 	    case '?':
 	    default:
@@ -246,7 +272,7 @@ checkout (int argc, char **argv)
 	if (!tag && !date)
 	    error (1, 0, "must specify a tag or date");
 
-	if (tag && isdigit ((unsigned char) tag[0]))
+	if (tag && isdigit (tag[0]))
 	    error (1, 0, "tag `%s' must be a symbolic tag", tag);
     }
 
@@ -257,8 +283,9 @@ checkout (int argc, char **argv)
     }
 #endif
 
-    if (!cat && !pipeout && !safe_location( where )) {
-        error(1, 0, "Cannot check out files into the repository itself");
+    if (!cat && !pipeout && !safe_location (where))
+    {
+        error (1, 0, "Cannot check out files into the repository itself");
     }
 
 #ifdef CLIENT_SUPPORT
@@ -291,27 +318,27 @@ checkout (int argc, char **argv)
 	if (!force_tag_match)
 	    send_arg ("-f");
 	if (aflag)
-	    send_arg("-A");
+	    send_arg ("-A");
 	if (!shorten)
-	    send_arg("-N");
+	    send_arg ("-N");
 	if (checkout_prune_dirs && m_type == CHECKOUT)
-	    send_arg("-P");
+	    send_arg ("-P");
 	client_prune_dirs = checkout_prune_dirs;
 	if (cat && !status)
-	    send_arg("-c");
+	    send_arg ("-c");
 	if (where != NULL)
 	    option_with_arg ("-d", where);
 	if (status)
-	    send_arg("-s");
+	    send_arg ("-s");
 	if (options != NULL && options[0] != '\0')
 	    send_arg (options);
 	option_with_arg ("-r", tag);
 	if (date)
 	    client_senddate (date);
-	if (join_rev1 != NULL)
-	    option_with_arg ("-j", join_rev1);
-	if (join_rev2 != NULL)
-	    option_with_arg ("-j", join_rev2);
+	if (join_orig1)
+	    option_with_arg ("-j", join_orig1);
+	if (join_orig2)
+	    option_with_arg ("-j", join_orig2);
 	send_arg ("--");
 
 	if (expand_modules)
@@ -363,10 +390,7 @@ checkout (int argc, char **argv)
 	else if (!tag)
 	    history_name = date;
 	else
-	{
-	    history_name = xmalloc (strlen (tag) + strlen (date) + 2);
-	    sprintf (history_name, "%s:%s", tag, date);
-	}
+	    history_name = Xasprintf ("%s:%s", tag, date);
     }
 
 
@@ -423,17 +447,15 @@ safe_location (char *where)
     TRACE (TRACE_FUNCTION, "safe_location( where=%s )",
            where ? where : "(null)");
 
-#ifdef CLIENT_SUPPORT
     /* Don't compare remote CVSROOTs to our destination directory. */
     if (current_parsed_root->isremote) return 1;
-#endif /* CLIENT_SUPPORT */
 
     /* set current - even if where is set we'll need to cd back... */
     current = xgetcwd ();
     if (current == NULL)
 	error (1, errno, "could not get working directory");
 
-    hardpath = xresolvepath (current_parsed_root->directory);
+    hardpath = xcanonicalize_file_name (current_parsed_root->directory);
 
     /* if where is set, set current to as much of where as exists,
      * or fail.
@@ -461,20 +483,12 @@ safe_location (char *where)
 	    }
 	    else if (errno == ENOENT)
 	    {
-		/* where_this_pass - last_component (where_this_pass) */
-		char *parent;
-
-		/* It's okay to cast out the const below since we know we
-		 * allocated where_this_pass and have control of it.
+		/* walk up the tree by setting
+		 * WHERE_THIS_PASS = WHERE_THIS_PASS
+	         *                   - last_component (WHERE_THIS_PASS)
 		 */
-		if ((parent = (char *)last_component (where_this_pass))
-		        != where_this_pass)
-		{
-		    /* strip the last_component */
-		    parent[-1] = '\0';
-		    /* continue */
-		}
-		else
+		char *parent = last_component (where_this_pass);
+		if (parent == where_this_pass)
 		{
 		    /* ERRNO == ENOENT
 		     *   && last_component (where_this_pass) == where_this_pass
@@ -485,6 +499,22 @@ safe_location (char *where)
 		    free (where_this_pass);
 		    break;
 		}
+		else if (!*parent)
+		{
+		    /* Under UNIX, it shouldn't be possible for cd'ing to a
+		     * root directory to return ENOENT, but under Windows, it
+		     * might (e.g. `chdir ("F:\\")', with F: unmounted).
+		     */
+		    error (1, errno, "\
+could not change directory to requested checkout directory `%s'",
+			   where_this_pass);
+		}
+		else
+		{
+		    /* strip the last_component */
+		    parent[-1] = '\0';
+		    /* continue */
+		}
 	    }
 	    else
 		/* we don't know how to handle other errors, so fail */
@@ -564,7 +594,7 @@ build_one_dir (char *repository, char *dirpath, int sticky)
 
 	if (!noexec)
 	{
-	    fp = open_file (CVSADM_ENTSTAT, "w+");
+	    fp = xfopen (CVSADM_ENTSTAT, "w+");
 	    if (fclose (fp) == EOF)
 		error (1, errno, "cannot close %s", CVSADM_ENTSTAT);
 #ifdef SERVER_SUPPORT
@@ -689,7 +719,7 @@ checkout_proc (int argc, char **argv, char *where_orig, char *mwhere,
            directory, let the user override it with the command-line
            -d option. */
 
-	if ((mwhere != NULL) && (! isabsolute (mwhere)))
+	if (mwhere && !ISABSOLUTE (mwhere))
 	    (void) strcat (where, mwhere);
 	else
 	    (void) strcat (where, argv[0]);
@@ -742,8 +772,7 @@ checkout_proc (int argc, char **argv, char *where_orig, char *mwhere,
 
 	/* Now mfile is a single path element. */
 
-	path = xmalloc (strlen (repository) + strlen (mfile) + 5);
-	(void) sprintf (path, "%s/%s", repository, mfile);
+	path = Xasprintf ("%s/%s", repository, mfile);
 	if (isdir (path))
 	{
 	    /* It's a directory, so tack it on to repository and
@@ -887,8 +916,6 @@ internal error: %s doesn't start with %s in checkout_proc",
 			   reposcopy);
 			   
 		*rp = '\0';
-		new->repository = xmalloc (strlen (reposcopy) + 5);
-		(void) strcpy (new->repository, reposcopy);
 		    
 		if (strcmp (reposcopy, current_parsed_root->directory) == 0)
 		{
@@ -897,8 +924,10 @@ internal error: %s doesn't start with %s in checkout_proc",
 		       is important).  We might be able to get rid
 		       of this after the we check out the other
 		       code that handles repository names. */
-		    (void) strcat (new->repository, "/.");
+		    new-> repository = Xasprintf ("%s/.", reposcopy);
 		}
+		else
+		    new->repository = xstrdup (reposcopy);
 	    }
 	}
 
@@ -916,7 +945,7 @@ internal error: %s doesn't start with %s in checkout_proc",
 	   build_one_dir whenever the -d command option was specified
 	   to checkout.  */
 
-	if (!isabsolute (where) && config->top_level_admin
+	if (!ISABSOLUTE (where) && config->top_level_admin
 	    && m_type == CHECKOUT)
 	{
 	    /* It may be argued that we shouldn't set any sticky
@@ -963,7 +992,7 @@ internal error: %s doesn't start with %s in checkout_proc",
 
 		Create_Admin (".", preload_update_dir, repository,
 			      NULL, NULL, 0, 0, m_type == CHECKOUT);
-		fp = open_file (CVSADM_ENTSTAT, "w+");
+		fp = xfopen (CVSADM_ENTSTAT, "w+");
 		if (fclose (fp) == EOF)
 		    error (1, errno, "cannot close %s", CVSADM_ENTSTAT);
 #ifdef SERVER_SUPPORT
@@ -1029,35 +1058,36 @@ internal error: %s doesn't start with %s in checkout_proc",
 	    goto out;
 	}
 	which = W_REPOS;
-	if (tag != NULL && !tag_validated)
+	if (tag && !tag_validated)
 	{
-	    tag_check_valid (tag, argc - 1, argv + 1, 0, aflag, NULL, false);
-	    tag_validated = 1;
+	    tag_check_valid (tag, argc - 1, argv + 1, 0, aflag,
+			     repository, false);
+	    tag_validated = true;
 	}
     }
     else
     {
 	which = W_LOCAL | W_REPOS;
-	if (tag != NULL && !tag_validated)
+	if (tag && !tag_validated)
 	{
 	    tag_check_valid (tag, argc - 1, argv + 1, 0, aflag,
 			     repository, false);
-	    tag_validated = 1;
+	    tag_validated = true;
 	}
     }
 
-    if (tag != NULL || date != NULL || join_rev1 != NULL)
+    if (tag || date || join_rev1 || join_date2)
 	which |= W_ATTIC;
 
-    if (! join_tags_validated)
+    if (!join_tags_validated)
     {
-        if (join_rev1 != NULL)
-	    tag_check_valid_join (join_rev1, argc - 1, argv + 1, 0, aflag,
-				  repository);
-	if (join_rev2 != NULL)
-	    tag_check_valid_join (join_rev2, argc - 1, argv + 1, 0, aflag,
-				  repository);
-	join_tags_validated = 1;
+        if (join_rev1)
+	    tag_check_valid (join_rev1, argc - 1, argv + 1, 0, aflag,
+			     repository, false);
+	if (join_rev2)
+	    tag_check_valid (join_rev2, argc - 1, argv + 1, 0, aflag,
+			     repository, false);
+	join_tags_validated = true;
     }
 
     /*
@@ -1073,55 +1103,13 @@ internal error: %s doesn't start with %s in checkout_proc",
 	err += do_update (0, NULL, options, tag, date,
 			  force_tag_match, false /* !local */ ,
 			  true /* update -d */ , aflag, checkout_prune_dirs,
-			  pipeout, which, join_rev1, join_rev2,
+			  pipeout, which, join_rev1, join_date1,
+			  join_rev2, join_date2,
 			  preload_update_dir, m_type == CHECKOUT,
 			  repository);
 	goto out;
     }
 
-    if (!pipeout)
-    {
-	int i;
-	List *entries;
-
-	/* we are only doing files, so register them */
-	entries = Entries_Open (0, NULL);
-	for (i = 1; i < argc; i++)
-	{
-	    char *line;
-	    Vers_TS *vers;
-	    struct file_info finfo;
-
-	    memset (&finfo, 0, sizeof finfo);
-	    finfo.file = argv[i];
-	    /* Shouldn't be used, so set to arbitrary value.  */
-	    finfo.update_dir = NULL;
-	    finfo.fullname = argv[i];
-	    finfo.repository = repository;
-	    finfo.entries = entries;
-	    /* The rcs slot is needed to get the options from the RCS
-               file */
-	    finfo.rcs = RCS_parse (finfo.file, repository);
-
-	    vers = Version_TS (&finfo, options, tag, date,
-			       force_tag_match, 0);
-	    if (vers->ts_user == NULL)
-	    {
-		line = xmalloc (strlen (finfo.file) + 15);
-		(void) sprintf (line, "Initial %s", finfo.file);
-		Register (entries, finfo.file,
-			  vers->vn_rcs ? vers->vn_rcs : "0",
-			  line, vers->options, vers->tag,
-			  vers->date, NULL);
-		free (line);
-	    }
-	    freevers_ts (&vers);
-	    freercsnode (&finfo.rcs);
-	}
-
-	Entries_Close (entries);
-    }
-
     /* Don't log "export", just regular "checkouts" */
     if (m_type == CHECKOUT && !pipeout)
 	history_write ('O', preload_update_dir, history_name, where,
@@ -1131,8 +1119,8 @@ internal error: %s doesn't start with %s in checkout_proc",
     err += do_update (argc - 1, argv + 1, options, tag, date,
 		      force_tag_match, local_specified, true /* update -d */,
 		      aflag, checkout_prune_dirs, pipeout, which, join_rev1,
-		      join_rev2, preload_update_dir, m_type == CHECKOUT,
-		      repository);
+		      join_date1, join_rev2, join_date2, preload_update_dir,
+		      m_type == CHECKOUT, repository);
 out:
     free (preload_update_dir);
     preload_update_dir = oldupdate;
@@ -1166,12 +1154,8 @@ emptydir_name (void)
 {
     char *repository;
 
-    repository = xmalloc (strlen (current_parsed_root->directory) 
-			  + sizeof (CVSROOTADM)
-			  + sizeof (CVSNULLREPOS)
-			  + 3);
-    (void) sprintf (repository, "%s/%s/%s", current_parsed_root->directory,
-		    CVSROOTADM, CVSNULLREPOS);
+    repository = Xasprintf ("%s/%s/%s", current_parsed_root->directory,
+			    CVSROOTADM, CVSNULLREPOS);
     if (!isfile (repository))
     {
 	mode_t omask;
@@ -1217,14 +1201,22 @@ build_dirs_and_chdir (struct dir_to_build *dirs, int sticky)
     while (dirs != NULL)
     {
 	const char *dir = last_component (dirs->dirpath);
-	int made_dir = 0;
+	bool made_dir = false;
 
-	made_dir = !mkdir_if_needed (dir);
-	if (made_dir) Subdir_Register (NULL, NULL, dir);
+	if (*dir)
+	{
+	    made_dir = !mkdir_if_needed (dir);
+	    if (made_dir) Subdir_Register (NULL, NULL, dir);
+	}
+	else
+	    /* Assume the (or a, under Windows) root directory need not be
+	     * created.
+	     */
+	    dir = dirs->dirpath;
 
 	if (CVS_CHDIR (dir) < 0)
 	{
-	    error (0, errno, "cannot chdir to %s", dir);
+	    error (0, errno, "cannot chdir to `%s'", dir);
 	    retval = 1;
 	    goto out;
 	}
