@@ -1,5 +1,27 @@
+/*
+ * Copyright (C) 1994-2006 The Free Software Foundation, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ */
+
 /* Interface between the client and the rest of CVS.  */
 
+#ifndef CLIENT_H
+#define CLIENT_H
+
+#include <sys/types.h>
+#include "root.h"
+
+
+
 /* Stuff shared with the server.  */
 char *mode_to_string (mode_t);
 int change_mode (const char *, const char *, int);
@@ -98,15 +120,15 @@ send_file_names (int argc, char **argv, unsigned int flags);
  * the files to operate on (or empty for everything), not options.
  * local is nonzero if we should not recurse (-l option).
  */
-void
-send_files (int argc, char **argv, int local, int aflag,
-		  unsigned int flags);
+void send_files (int argc, char **argv, int local, int aflag,
+		 unsigned int flags);
 
 /* Flags for send_files.  */
-# define SEND_BUILD_DIRS 1
-# define SEND_FORCE 2
-# define SEND_NO_CONTENTS 4
-# define BACKUP_MODIFIED_FILES 8
+# define SEND_BUILD_DIRS	(1 << 0)
+# define SEND_FORCE		(1 << 1)
+# define SEND_NO_CONTENTS	(1 << 2)
+# define BACKUP_MODIFIED_FILES	(1 << 3)
+# define FORCE_SIGNATURES	(1 << 4)
 
 /* Send an argument to the remote server.  */
 void
@@ -114,7 +136,7 @@ send_arg (const char *string);
 
 /* Send a string of single-char options to the remote server, one by one.  */
 void
-send_option_string (const char *string);
+send_options (int argc, char * const *argv);
 
 void send_a_repository (const char *, const char *, const char *);
 
@@ -195,4 +217,15 @@ int client_process_import_file
 void client_import_done (void);
 void client_notify (const char *, const char *, const char *, int,
                     const char *);
+
+#if defined AUTH_CLIENT_SUPPORT || defined HAVE_KERBEROS || defined HAVE_GSSAPI
+# include <sys/socket.h>
+# include <netinet/in.h>
+# include <arpa/inet.h>
+# include <netdb.h>
+struct hostent *init_sockaddr (struct sockaddr_in *, char *, unsigned int);
+#endif
+
 #endif /* CLIENT_SUPPORT */
+
+#endif /* CLIENT_H */
