@@ -1,6 +1,11 @@
 /*
- * Copyright (c) 1992, Brian Berliner and Jeff Polk
- * Copyright (c) 1989-1992, Brian Berliner
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
+ *
+ * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
+ *                                  and others.
+ *
+ * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
+ * Portions Copyright (C) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -11,6 +16,13 @@
  * "Entries" file is prefilled from the "initrecord" argument.
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* CVS Headers.  */
+#include "repos.h"
+
 #include "cvs.h"
 
 
@@ -33,20 +45,22 @@ Create_Admin (const char *dir, const char *update_dir, const char *repository,
     char *reposcopy;
     char *tmp;
 
-    TRACE ( 1, "Create_Admin (%s, %s, %s, %s, %s, %d, %d, %d)",
-	    dir, update_dir, repository, tag ? tag : "",
-	    date ? date : "", nonbranch, warn, dotemplate );
+    TRACE (TRACE_FUNCTION, "Create_Admin (%s, %s, %s, %s, %s, %d, %d, %d)",
+	   dir, update_dir, repository, tag ? tag : "",
+	   date ? date : "", nonbranch, warn, dotemplate);
 
     if (noexec)
 	return 0;
 
-    tmp = xmalloc (strlen (dir) + 100);
-    (void) sprintf (tmp, "%s/%s", dir, CVSADM);
+    tmp = Xasprintf ("%s/%s", dir, CVSADM);
     if (isfile (tmp))
 	error (1, 0, "there is a version in %s already", update_dir);
 
     if (CVS_MKDIR (tmp, 0777) < 0)
     {
+	free (tmp);
+	tmp = NULL;
+
 	/* We want to print out the entire update_dir, since a lot of
 	   our code calls this function with dir == "." or dir ==
 	   NULL.  I hope that gives enough information in cases like
@@ -62,21 +76,25 @@ Create_Admin (const char *dir, const char *update_dir, const char *repository,
 	       the warning at least we let them know what is going on.  */
 	    error (0, errno, "warning: cannot make directory %s in %s",
 		   CVSADM, update_dir);
-	    free (tmp);
 	    return 1;
 	}
 	else
 	    error (1, errno, "cannot make directory %s in %s",
 		   CVSADM, update_dir);
     }
+    else
+    {
+	free (tmp);
+	tmp = NULL;
+    }
 
     /* record the current cvs root for later use */
 
     Create_Root (dir, original_parsed_root->original);
     if (dir != NULL)
-	(void) sprintf (tmp, "%s/%s", dir, CVSADM_REP);
+	tmp = Xasprintf ("%s/%s", dir, CVSADM_REP);
     else
-	(void) strcpy (tmp, CVSADM_REP);
+	tmp = xstrdup (CVSADM_REP);
     fout = CVS_FOPEN (tmp, "w+");
     if (fout == NULL)
     {
@@ -107,12 +125,10 @@ Create_Admin (const char *dir, const char *update_dir, const char *repository,
      * the leading CVSroot argument.
      */
     {
-    char *path = xmalloc (strlen (current_parsed_root->directory) + 2);
-
-    (void) sprintf (path, "%s/", current_parsed_root->directory);
-    if (strncmp (cp, path, strlen (path)) == 0)
-	cp += strlen (path);
-    free (path);
+	char *path = Xasprintf ("%s/", current_parsed_root->directory);
+	if (strncmp (cp, path, strlen (path)) == 0)
+	    cp += strlen (path);
+	free (path);
     }
 
     if (fprintf (fout, "%s\n", cp) < 0)
@@ -154,7 +170,7 @@ Create_Admin (const char *dir, const char *update_dir, const char *repository,
     /* Create a new CVS/Tag file */
     WriteTag (dir, tag, date, nonbranch, update_dir, repository);
 
-    TRACE (1, "Create_Admin");
+    TRACE (TRACE_FUNCTION, "Create_Admin");
 
     free (reposcopy);
     free (tmp);
