@@ -17,29 +17,36 @@
    definitions under operating systems (like, say, Windows NT) with different
    file system semantics.  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* Verify interface.  */
+#include "filesubr.h"
+
+/* GNULIB */
 #include "lstat.h"
 #include "save-cwd.h"
 #include "xsize.h"
 
+/* CVS */
+#include "cvs.h"
+
 static int deep_remove_dir (const char *path);
 
+
+
 /*
- * Copies "from" to "to".
+ * Copies FROM to TO.  Ignores NOEXEC.
  */
 void
-copy_file (const char *from, const char *to)
+force_copy_file (const char *from, const char *to)
 {
     struct stat sb;
     struct utimbuf t;
     int fdin, fdout;
     ssize_t rsize;
 
-    TRACE (TRACE_FUNCTION, "copy(%s,%s)", from, to);
-
-    if (noexec)
-	return;
-
     /* If the file to be copied is a link or a device, then just create
        the new link or device appropriately. */
     if ((rsize = islink (from)) > 0)
@@ -73,10 +80,10 @@ copy_file (const char *from, const char *to)
 	{
 	    char buf[BUFSIZ];
 	    int n;
-	    
-	    for (;;) 
+
+	    for (;;)
 	    {
-		n = read (fdin, buf, sizeof(buf));
+		n = read (fdin, buf, sizeof (buf));
 		if (n == -1)
 		{
 #ifdef EINTR
@@ -85,16 +92,15 @@ copy_file (const char *from, const char *to)
 #endif
 		    error (1, errno, "cannot read file %s for copying", from);
 		}
-		else if (n == 0) 
+		else if (n == 0)
 		    break;
-		
-		if (write(fdout, buf, n) != n) {
+
+		if (write (fdout, buf, n) != n)
 		    error (1, errno, "cannot write file %s for copying", to);
-		}
 	    }
 	}
 
-	if (close (fdin) < 0) 
+	if (close (fdin) < 0)
 	    error (0, errno, "cannot close %s", from);
 	if (close (fdout) < 0)
 	    error (1, errno, "cannot close %s", to);
@@ -109,6 +115,22 @@ copy_file (const char *from, const char *to)
 
 
 
+/*
+ * Copies FROM to TO.  Honors NOEXEC.
+ */
+void
+copy_file (const char *from, const char *to)
+{
+    TRACE (TRACE_FUNCTION, "copy (%s, %s)", from, to);
+
+    if (noexec)
+	return;
+
+    force_copy_file (from, to);
+}
+
+
+
 /* FIXME-krp: these functions would benefit from caching the char * &
    stat buf.  */
 
@@ -131,6 +153,11 @@ isdir (const char *file)
 /*
  * Returns 0 if the argument file is not a symbolic link.
  * Returns size of the link if it is a symbolic link.
+ *
+ * FIXME: Is there a good reason that the off_t specified by POSIX for st_size
+ *        (http://www.opengroup.org/susv3xbd/sys/stat.h.html) is converted to
+ *        ssize_t here?  rcs.h uses off_t, so it's not because off_t isn't
+ *        portable.
  */
 ssize_t
 islink (const char *file)
@@ -218,13 +245,13 @@ isaccessible (const char *file, const int mode)
     int gmask = 0;
     int omask = 0;
     int uid, mask;
-    
-    if (stat (file, &sb)== -1)
+
+    if (stat (file, &sb) == -1)
 	return false;
     if (mode == F_OK)
 	return true;
 
-    uid = geteuid();
+    uid = geteuid ();
     if (uid == 0)		/* superuser */
     {
 	if (!(mode & X_OK) || (sb.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH)))
@@ -233,7 +260,7 @@ isaccessible (const char *file, const int mode)
 	errno = EACCES;
 	return false;
     }
-	
+
     if (mode & R_OK)
     {
 	umask |= S_IRUSR;
@@ -253,7 +280,7 @@ isaccessible (const char *file, const int mode)
 	omask |= S_IXOTH;
     }
 
-    mask = sb.st_uid == uid ? umask : sb.st_gid == getegid() ? gmask : omask;
+    mask = sb.st_uid == uid ? umask : sb.st_gid == getegid () ? gmask : omask;
     if ((sb.st_mode & mask) == mask)
 	return true;
     errno = EACCES;
@@ -324,6 +351,8 @@ mkdir_if_needed (const char *name)
     return 0;
 }
 
+
+
 /*
  * Change the mode of a file, either adding write permissions, or removing
  * all write permissions.  Either change honors the current umask setting.
@@ -331,8 +360,8 @@ mkdir_if_needed (const char *name)
  * Don't do anything if PreservePermissions is set to `yes'.  This may
  * have unexpected consequences for some uses of xchmod.
  */
-void
-xchmod (const char *fname, int writable)
+static void
+ixchmod (const char *fname, bool writable, bool noexec)
 {
     struct stat sb;
     mode_t mode, oumask;
@@ -351,16 +380,12 @@ xchmod (const char *fname, int writable)
     oumask = umask (0);
     (void) umask (oumask);
     if (writable)
-    {
 	mode = sb.st_mode | (~oumask
 			     & (((sb.st_mode & S_IRUSR) ? S_IWUSR : 0)
 				| ((sb.st_mode & S_IRGRP) ? S_IWGRP : 0)
 				| ((sb.st_mode & S_IROTH) ? S_IWOTH : 0)));
-    }
     else
-    {
 	mode = sb.st_mode & ~(S_IWRITE | S_IWGRP | S_IWOTH) & ~oumask;
-    }
 
     TRACE (TRACE_FUNCTION, "chmod(%s,%o)", fname, (unsigned int) mode);
 
@@ -371,6 +396,26 @@ xchmod (const char *fname, int writable)
 	error (0, errno, "cannot change mode of file %s", fname);
 }
 
+
+
+/* See description for ixchmod.  Ignores NOEXEC.  */
+void
+force_xchmod (const char *fname, bool writable)
+{
+    ixchmod (fname, writable, false);
+}
+
+
+
+/* See description for ixchmod.  Honors NOEXEC.  */
+void
+xchmod (const char *fname, bool writable)
+{
+    ixchmod (fname, writable, noexec);
+}
+
+
+
 /*
  * Rename a file and die if it fails
  */
@@ -387,7 +432,8 @@ rename_file (const char *from, const char *to)
 }
 
 /*
- * unlink a file, if possible.
+ * unlink a file, if possible.  Use CVS_UNLINK if you want to ignore the
+ * noexec flag.
  */
 int
 unlink_file (const char *f)
@@ -395,9 +441,9 @@ unlink_file (const char *f)
     TRACE (TRACE_FUNCTION, "unlink_file(%s)", f);
 
     if (noexec)
-	return (0);
+	return 0;
 
-    return (CVS_UNLINK (f));
+    return CVS_UNLINK (f);
 }
 
 
@@ -484,7 +530,7 @@ deep_remove_dir (const char *path)
 		/* See comment in unlink_file_dir explanation of why we use
 		   isdir instead of just calling unlink and checking the
 		   status.  */
-		if (isdir (buf)) 
+		if (isdir (buf))
 		{
 		    if (deep_remove_dir (buf))
 		    {
@@ -535,7 +581,7 @@ block_read (int fd, char *buf, size_t nchars)
     char *bp = buf;
     size_t nread;
 
-    do 
+    do
     {
 	nread = read (fd, bp, nchars);
 	if (nread == (size_t)-1)
@@ -548,7 +594,7 @@ block_read (int fd, char *buf, size_t nchars)
 	}
 
 	if (nread == 0)
-	    break; 
+	    break;
 
 	bp += nread;
 	nchars -= nread;
@@ -557,7 +603,7 @@ block_read (int fd, char *buf, size_t nchars)
     return bp - buf;
 } 
 
-    
+
 /*
  * Compare "file1" to "file2". Return non-zero if they don't compare exactly.
  * If FILE1 and FILE2 are special files, compare their salient characteristics
@@ -615,7 +661,7 @@ xcmp (const char *file1, const char *file2)
     if ((fd2 = open (file2, O_RDONLY)) < 0)
 	error (1, errno, "cannot open file %s for comparing", file2);
 
-    /* A generic file compare routine might compare st_dev & st_ino here 
+    /* A generic file compare routine might compare st_dev & st_ino here
        to see if the two files being compared are actually the same file.
        But that won't happen in CVS, so we won't bother. */
 
@@ -634,7 +680,7 @@ xcmp (const char *file1, const char *file2)
 	buf1 = xmalloc (buf_size);
 	buf2 = xmalloc (buf_size);
 
-	do 
+	do
 	{
 	    read1 = block_read (fd1, buf1, buf_size);
 	    if (read1 == (size_t)-1)
@@ -646,16 +692,16 @@ xcmp (const char *file1, const char *file2)
 
 	    /* assert (read1 == read2); */
 
-	    ret = memcmp(buf1, buf2, read1);
+	    ret = memcmp (buf1, buf2, read1);
 	} while (ret == 0 && read1 == buf_size);
 
 	free (buf1);
 	free (buf2);
     }
-	
+
     (void) close (fd1);
     (void) close (fd2);
-    return (ret);
+    return ret;
 }
 
 /* Generate a unique temporary filename.  Returns a pointer to a newly
@@ -739,7 +785,10 @@ cvs_temp_file (char **filename)
     }
 
     if (fp == NULL)
+    {
 	free (fn);
+	fn = NULL;
+    }
 
     /* mkstemp is defined to open mode 0600 using glibc 2.0.7+.  There used
      * to be a complicated #ifdef checking the library versions here and then
@@ -756,25 +805,16 @@ cvs_temp_file (char **filename)
      */
 
     *filename = fn;
+    if (fn == NULL && fp != NULL)
+    {
+	fclose (fp);
+	fp = NULL;
+    }
     return fp;
 }
 
 
 
-/* Return a pointer into PATH's last component.  */
-const char *
-last_component (const char *path)
-{
-    const char *last = strrchr (path, '/');
-    
-    if (last && (last != path))
-        return last + 1;
-    else
-        return path;
-}
-
-
-
 /* Return the home directory.  Returns a pointer to storage
    managed by this function or its callees (currently getenv).
    This function will return the same thing every time it is
@@ -842,7 +882,9 @@ void
 expand_wild (int argc, char **argv, int *pargc, char ***pargv)
 {
     int i;
-    if (size_overflow_p (xtimes (argc, sizeof (char *)))) {
+    assert (argv || !argc);
+    if (size_overflow_p (xtimes (argc, sizeof (char *))))
+    {
 	*pargc = 0;
 	*pargv = NULL;
 	error (0, 0, "expand_wild: too many arguments");
