@@ -1,19 +1,38 @@
-/* This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.  */
+/*
+ * Copyright (C) 2006 The Free Software Foundation, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ */
 
 /*
  * .cvsignore file support contributed by David G. Grubbs <dgg@odi.com>
  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* Verify interface.  */
+#include "ignore.h"
+
+/* GNULIB headers.  */
 #include "getline.h"
+#include "lstat.h"
+
+/* CVS headers.  */
+#include "wrapper.h"
+
+#include "cvs.h"
+
+
 
 /*
  * Ignore file section.
@@ -65,13 +84,11 @@ ign_setup (void)
     ign_add (tmp, 0);
     free (tmp);
 
-#ifdef CLIENT_SUPPORT
     /* The client handles another way, by (after it does its own ignore file
        processing, and only if !ign_inhibit_server), letting the server
        know about the files and letting it decide whether to ignore
        them based on CVSROOOTADM_IGNORE.  */
     if (!current_parsed_root->isremote)
-#endif
     {
 	char *file = Xasprintf ("%s/%s/%s", current_parsed_root->directory,
 				CVSROOTADM, CVSROOTADM_IGNORE);
@@ -433,7 +450,7 @@ ignore_files (List *ilist, List *entries, const char *update_dir,
 #ifdef DT_DIR
 	    dp->d_type != DT_UNKNOWN ||
 #endif
-	    CVS_LSTAT (file, &sb) != -1)
+	    lstat (file, &sb) != -1)
 	{
 
 	    if (
