@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
  * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
  *                                  and others.
@@ -21,9 +21,24 @@
  * Additional arguments specify more Vendor Release Tags.
  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* GNULIB headers.  */
+#include "base64.h"
+#include "lstat.h"
 #include "save-cwd.h"
 
+/* CVS headers.  */
+#include "ignore.h"
+#include "logmsg.h"
+#include "wrapper.h"
+
+#include "cvs.h"
+
+
+
 static char *get_comment (const char *user);
 static int add_rev (char *message, RCSNode *rcs, char *vfile,
 			  char *vers);
@@ -104,17 +119,14 @@ import (int argc, char **argv)
 	{
 	    case 'Q':
 	    case 'q':
-#ifdef SERVER_SUPPORT
 		/* The CVS 1.5 client sends these options (in addition to
 		   Global_option requests), so we must ignore them.  */
 		if (!server_active)
-#endif
 		    error (1, 0,
 			   "-q or -Q must be specified before \"%s\"",
 			   cvs_cmd_name);
 		break;
 	    case 'd':
-#ifdef SERVER_SUPPORT
 		if (server_active)
 		{
 		    /* CVS 1.10 and older clients will send this, but it
@@ -124,7 +136,6 @@ import (int argc, char **argv)
 			   "warning: not setting the time of import from the file");
 		    error (0, 0, "due to client limitations");
 		}
-#endif
 		use_file_modtime = 1;
 		break;
 	    case 'b':
@@ -167,7 +178,6 @@ import (int argc, char **argv)
     if (argc < 3)
 	usage (import_usage);
 
-#ifdef SERVER_SUPPORT
     /* This is for handling the Checkin-time request.  It might seem a
        bit odd to enable the use_file_modtime code even in the case
        where Checkin-time was not sent for a particular file.  The
@@ -179,7 +189,6 @@ import (int argc, char **argv)
 
     if (server_active)
 	use_file_modtime = 1;
-#endif
 
     /* Don't allow "CVS" as any directory in module path.
      *
@@ -206,7 +215,7 @@ import (int argc, char **argv)
 		error (1, 0, "tag `%s' was specified more than once", argv[i]);
     }
 
-    if (isabsolute (argv[0]) || pathname_levels (argv[0]) > 0)
+    if (ISABSOLUTE (argv[0]) || pathname_levels (argv[0]) > 0)
 	/* It is somewhere between a security hole and "unexpected" to
 	   let the client start mucking around outside the cvsroot
 	   (wouldn't get the right CVSROOT configuration, &c).  */
@@ -257,20 +266,12 @@ import (int argc, char **argv)
     }
 #endif
 
-    if (
-#ifdef SERVER_SUPPORT
-        !server_active &&
-#endif
-        use_editor)
+    if (!server_active && use_editor)
     {
 	do_editor (NULL, &message,
-#ifdef CLIENT_SUPPORT
-		   current_parsed_root->isremote ? NULL :
-#endif
-			repository,
+		   current_parsed_root->isremote ? NULL : repository,
 		   NULL);
     }
-    do_verify (&message, repository);
     msglen = message == NULL ? 0 : strlen (message);
     if (msglen == 0 || message[msglen - 1] != '\n')
     {
@@ -333,6 +334,19 @@ import (int argc, char **argv)
 	error (1, 0, "attempt to import the repository");
     }
 
+    ulist = getlist ();
+    p = getnode ();
+    p->type = UPDATE;
+    p->delproc = update_delproc;
+    p->key = xstrdup ("- Imported sources");
+    li = xmalloc (sizeof (struct logfile_info));
+    li->type = T_TITLE;
+    li->tag = xstrdup (vbranch);
+    li->rev_old = li->rev_new = NULL;
+    p->data = li;
+    (void) addnode (ulist, p);
+    do_verify (&message, repository, ulist);
+
     /*
      * Make all newly created directories writable.  Should really use a more
      * sophisticated security mechanism here.
@@ -342,7 +356,8 @@ import (int argc, char **argv)
 
     /* Create the logfile that will be logged upon completion */
     if ((logfp = cvs_temp_file (&tmpfile)) == NULL)
-	error (1, errno, "cannot create temporary file `%s'", tmpfile);
+	error (1, errno, "cannot create temporary file `%s'",
+	       tmpfile ? tmpfile : "(null)");
     /* On systems where we can unlink an open file, do so, so it will go
        away no matter how we exit.  FIXME-maybe: Should be checking for
        errors but I'm not sure which error(s) we get if we are on a system
@@ -419,17 +434,6 @@ import (int argc, char **argv)
     /*
      * Write out the logfile and clean up.
      */
-    ulist = getlist ();
-    p = getnode ();
-    p->type = UPDATE;
-    p->delproc = update_delproc;
-    p->key = xstrdup ("- Imported sources");
-    li = xmalloc (sizeof (struct logfile_info));
-    li->type = T_TITLE;
-    li->tag = xstrdup (vbranch);
-    li->rev_old = li->rev_new = NULL;
-    p->data = li;
-    (void) addnode (ulist, p);
     Update_Logfile (repository, message, logfp, ulist);
     dellist (&ulist);
     if (fclose (logfp) < 0)
@@ -465,6 +469,9 @@ import_descend (char *message, char *vtag, int targc, char **targv)
     ign_add_file (CVSDOTIGNORE, 1);
     wrap_add_file (CVSDOTWRAPPER, 1);
 
+    if (!current_parsed_root->isremote)
+	lock_dir_for_write (repository);
+
     if ((dirp = CVS_OPENDIR (".")) == NULL)
     {
 	error (0, errno, "cannot open directory");
@@ -477,13 +484,19 @@ import_descend (char *message, char *vtag, int targc, char **targv)
 	{
 	    if (strcmp (dp->d_name, ".") == 0 || strcmp (dp->d_name, "..") == 0)
 		goto one_more_time_boys;
-#ifdef SERVER_SUPPORT
+
 	    /* CVS directories are created in the temp directory by
 	       server.c because it doesn't special-case import.  So
 	       don't print a message about them, regardless of -I!.  */
 	    if (server_active && strcmp (dp->d_name, CVSADM) == 0)
 		goto one_more_time_boys;
-#endif
+
+	    /* FIXME: .#filename.sig is where the server currently saves the
+	     * signature data when available.  For now just ignore it.
+	     */
+	    if (server_active && !fnmatch (".#*.sig", dp->d_name, 0))
+		goto one_more_time_boys;
+
 	    if (ign_name (dp->d_name))
 	    {
 		add_log ('I', dp->d_name);
@@ -547,6 +560,9 @@ import_descend (char *message, char *vtag, int targc, char **targv)
 	(void) CVS_CLOSEDIR (dirp);
     }
 
+    if (!current_parsed_root->isremote)
+	Simple_Lock_Cleanup ();
+
     if (dirlist != NULL)
     {
 	Node *head, *p;
@@ -707,7 +723,7 @@ update_rcs_file (char *message, char *vfile, char *vtag, int targc,
            not NULL?  */
 	expand = (vers->srcfile->expand != NULL
 		  && vers->srcfile->expand[0] == 'b') ? "-kb" : "-ko";
-	different = RCS_cmp_file (vers->srcfile, vers->vn_rcs, NULL,
+	different = RCS_cmp_file (vers->srcfile, vers->tag, vers->vn_rcs, NULL,
 	                          NULL, expand, vfile);
 	if (tocvsPath)
 	    if (unlink_file_dir (tocvsPath) < 0)
@@ -1059,6 +1075,11 @@ get_comment (const char *user)
  * RETURNS
  *   Return value is 0 for success, or nonzero for failure (in which
  *   case an error message will have already been printed).
+ *
+ * FIXME
+ *   I see very few reasons why this shoudn't be merged with RCS_rewrite ()
+ *   or better yet RCS_checkin () and I would guess this would ease
+ *   maintenance.
  */
 int
 add_rcs_file (const char *message, const char *rcs, const char *user,
@@ -1081,6 +1102,11 @@ add_rcs_file (const char *message, const char *rcs, const char *user,
     mode_t file_type;
     char *dead_revision = NULL;
 
+    TRACE (TRACE_FUNCTION,
+	   "add_rcs_file (`%s', `%s', `%s', `%s', `%s', `%s', `%s')",
+	   rcs, user, add_vhead, TRACE_NULL (key_opt),
+	   TRACE_NULL (add_vbranch), TRACE_NULL (vtag), TRACE_NULL (desctext));
+
     if (noexec)
 	return 0;
 
@@ -1145,7 +1171,7 @@ add_rcs_file (const char *message, const char *rcs, const char *user,
        `cannot lstat'.  I don't see a way around this, since we must
        stat the file before opening it. -twp */
 
-    if (CVS_LSTAT (userfile, &sb) < 0)
+    if (lstat (userfile, &sb) < 0)
     {
 	/* not fatal, continue import */
 	if (add_logfp != NULL)
@@ -1312,6 +1338,30 @@ add_rcs_file (const char *message, const char *rcs, const char *user,
 	if (fprintf (fprcs, "commitid        %s;\012", global_session_id) < 0)
 	    goto write_error;
 
+	if (!add_vbranch
+	    && (get_sign_commits (true) || have_sigfile (userfile)))
+	{
+	    char *rawsig;
+	    size_t rawlen;
+	    char *b64sig;
+
+	    TRACE (TRACE_DATA, "add_rcs_file: found signature.");
+
+	    rawsig = get_signature ("", userfile,
+				    key_opt && !strcmp (key_opt, "b"),
+				    &rawlen);
+	    base64_encode_alloc (rawsig, rawlen, &b64sig);
+	    if (!b64sig) xalloc_die ();
+	    free (rawsig);
+
+	    if (fprintf (fprcs, "openpgp-signatures        %s;\012",
+			 b64sig) < 0)
+		goto write_error;
+	    free (b64sig);
+	}
+	else
+	    TRACE (TRACE_DATA, "add_rcs_file: signature not found.");
+
 #ifdef PRESERVE_PERMISSIONS_SUPPORT
 	/* Store initial permissions if necessary. */
 	if (config->preserve_perms)
@@ -1322,7 +1372,7 @@ add_rcs_file (const char *message, const char *rcs, const char *user,
 	}
 #endif
 
-	if (add_vbranch != NULL)
+	if (add_vbranch)
 	{
 	    if (fprintf (fprcs, "\012%s.1\012", add_vbranch) < 0 ||
 		fprintf (fprcs, "date     %s;  author %s;  state Exp;\012",
@@ -1332,6 +1382,29 @@ add_rcs_file (const char *message, const char *rcs, const char *user,
 	        fprintf (fprcs, "commitid        %s;\012", global_session_id) < 0)
 		goto write_error;
 
+	    if (get_sign_commits (true) || have_sigfile (userfile))
+	    {
+		char *rawsig;
+		size_t rawlen;
+		char *b64sig;
+
+		TRACE (TRACE_DATA, "add_rcs_file: found signature.");
+
+		rawsig = get_signature ("", userfile,
+					key_opt && !strcmp (key_opt, "b"),
+					&rawlen);
+		base64_encode_alloc (rawsig, rawlen, &b64sig);
+		if (!b64sig) xalloc_die ();
+		free (rawsig);
+
+		if (fprintf (fprcs, "openpgp-signatures        %s;\012",
+			     b64sig) < 0)
+		    goto write_error;
+		free (b64sig);
+	    }
+	    else
+		TRACE (TRACE_DATA, "add_rcs_file: signature not found.");
+
 #ifdef PRESERVE_PERMISSIONS_SUPPORT
 	    /* Store initial permissions if necessary. */
 	    if (config->preserve_perms)
@@ -1732,11 +1805,7 @@ import_descend_dir (char *message, char *dir, char *vtag, int targc,
 	repository = new;
     }
 
-#ifdef CLIENT_SUPPORT
     if (!quiet && !current_parsed_root->isremote)
-#else
-    if (!quiet)
-#endif
 	error (0, 0, "Importing %s", repository);
 
     if (CVS_CHDIR (dir) < 0)
@@ -1747,11 +1816,7 @@ import_descend_dir (char *message, char *dir, char *vtag, int targc,
 	err = 1;
 	goto out;
     }
-#ifdef CLIENT_SUPPORT
     if (!current_parsed_root->isremote && !isdir (repository))
-#else
-    if (!isdir (repository))
-#endif
     {
 	rcs = Xasprintf ("%s%s", repository, RCSEXT);
 	if (isfile (repository) || isfile (rcs))
