@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
  * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
  *                                  and others.
@@ -17,9 +17,26 @@
  * (recursive by default).
  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h> 
+#endif
+
+/* ANSI C Headers.  */
 #include <assert.h>
 
+/* GNULIB Headers.  */
+#include "base64.h"
+
+/* CVS Headers.  */
+#include "gpg.h"
+#include "ignore.h"
+#include "recurse.h"
+#include "wrapper.h"
+
+#include "cvs.h"
+
+
+
 /* This structure holds information parsed from the -r option.  */
 
 struct option_revlist
@@ -310,6 +327,7 @@ cvslog (int argc, char **argv)
 	{
 	    p = log_data.datelist;
 	    log_data.datelist = p->next;
+	    assert (p->start && p->end);
 	    send_to_server ("Argument -d\012", 0);
 	    send_to_server ("Argument ", 0);
 	    date_to_internet (datetmp, p->start);
@@ -321,23 +339,21 @@ cvslog (int argc, char **argv)
 	    date_to_internet (datetmp, p->end);
 	    send_to_server (datetmp, 0);
 	    send_to_server ("\012", 0);
-	    if (p->start)
-		free (p->start);
-	    if (p->end)
-		free (p->end);
+	    free (p->start);
+	    free (p->end);
 	    free (p);
 	}
 	while (log_data.singledatelist != NULL)
 	{
 	    p = log_data.singledatelist;
 	    log_data.singledatelist = p->next;
+	    assert (p->end);
 	    send_to_server ("Argument -d\012", 0);
 	    send_to_server ("Argument ", 0);
 	    date_to_internet (datetmp, p->end);
 	    send_to_server (datetmp, 0);
 	    send_to_server ("\012", 0);
-	    if (p->end)
-		free (p->end);
+	    free (p->end);
 	    free (p);
 	}
 	    
@@ -1070,6 +1086,8 @@ log_expand_revlist (RCSNode *rcs, char *baserev,
 	{
 	    char *branch;
 
+	    assert (r->first);
+
 	    /* Print just the head of the branch.  */
 	    if (isdigit ((unsigned char) r->first[0]))
 		nr->first = RCS_getbranch (rcs, r->first, 1);
@@ -1669,6 +1687,42 @@ log_version (struct log_data *log_data, struct revlist *revlist, RCSNode *rcs,
 	cvs_output ("\n", 1);
     }
 
+    p = findnode (ver->other_delta, "openpgp-signatures");
+    if (p)
+    {
+	char *rawsig;
+	size_t rawsiglen;
+	struct buffer *membuf;
+	struct openpgp_signature sig;
+	int rc;
+
+	if (!base64_decode_alloc (p->data, p->len, &rawsig, &rawsiglen))
+	    error (0, 0, "Unable to base64-decode OpenPGP signature.");
+	if (!rawsig)
+	    xalloc_die ();
+
+	membuf = buf_nonio_initialize (NULL);
+	buf_output (membuf, rawsig, rawsiglen);
+
+	while (!(rc = parse_signature (membuf, &sig)))
+	{
+	    char *hexsig;
+	    cvs_output_tagged ("openpgp-keyid-header",
+			       "OpenPGP signature using key ID ");
+	    /* GnuPG truncates this too.  */
+	    hexsig = gpg_keyid2string (sig.keyid);
+	    cvs_output_tagged ("openpgp-keyid", hexsig);
+	    free (hexsig);
+	    cvs_output_tagged ("openpgp-keyid-footer", ";");
+	    cvs_output_tagged ("newline", NULL);
+	}
+
+	if (rc == -2)
+	    error (1, 0, "Memory allocation failure parsing signature.");
+
+	buf_free (membuf);
+    }
+
     p = findnode (ver->other, "log");
     /* The p->date == NULL case is the normal one for an empty log
        message (rcs-14 in sanity.sh).  I don't think the case where
