@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
  * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
  *                                  and others.
@@ -11,9 +11,22 @@
  * specified in the README file that comes with the CVS source distribution.
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* Verify interface.  */
+#include "logmsg.h"
+
+/* GNULIB Headers.  */
+#include "getline.h"
+
+/* CVS Headers.  */
+#include "repos.h"
 
 #include "cvs.h"
-#include "getline.h"
+
+
 
 static int find_type (Node * p, void *closure);
 static int fmt_proc (Node * p, void *closure);
@@ -47,6 +60,9 @@ struct verifymsg_proc_data
      * been changed when RereadLogAfterVerify is STAT.
      */
     struct stat pre_stbuf;
+   /* The list of files being changed, with new and old version numbers.
+    */
+   List *changes;
 };
 
 /*
@@ -199,11 +215,7 @@ do_editor (const char *dir, char **messagep, const char *repository,
     struct stat pre_stbuf, post_stbuf;
     int retcode = 0;
 
-#ifdef CLIENT_SUPPORT
     assert (!current_parsed_root->isremote != !repository);
-#else
-    assert (repository);
-#endif
 
     if (noexec || reuse_log_message)
 	return;
@@ -284,7 +296,7 @@ do_editor (const char *dir, char **messagep, const char *repository,
     /* finish off the temp file */
     if (fclose (fp) == EOF)
         error (1, errno, "%s", fname);
-    if ( CVS_STAT (fname, &pre_stbuf) == -1)
+    if (stat (fname, &pre_stbuf) == -1)
 	pre_stbuf.st_mtime = 0;
 
     /* run the editor */
@@ -301,7 +313,7 @@ do_editor (const char *dir, char **messagep, const char *repository,
     if (*messagep)
 	free (*messagep);
 
-    if ( CVS_STAT (fname, &post_stbuf) != 0)
+    if (stat (fname, &post_stbuf) != 0)
 	    error (1, errno, "cannot find size of temp file %s", fname);
 
     if (post_stbuf.st_size == 0)
@@ -399,17 +411,15 @@ do_editor (const char *dir, char **messagep, const char *repository,
    independant of the running of an editor for getting a message.
  */
 void
-do_verify (char **messagep, const char *repository)
+do_verify (char **messagep, const char *repository, List *changes)
 {
     int err;
     struct verifymsg_proc_data data;
     struct stat post_stbuf;
 
-#ifdef CLIENT_SUPPORT
-    if( current_parsed_root->isremote )
+    if (current_parsed_root->isremote)
 	/* The verification will happen on the server.  */
 	return;
-#endif
 
     /* FIXME? Do we really want to skip this on noexec?  What do we do
        for the other administrative files?  */
@@ -421,6 +431,7 @@ do_verify (char **messagep, const char *repository)
 
     data.message = *messagep;
     data.fname = NULL;
+    data.changes = changes;
     if ((err = Parse_Info (CVSROOTADM_VERIFYMSG, repository,
 	                  verifymsg_proc, 0, &data)) != 0)
     {
@@ -446,7 +457,7 @@ do_verify (char **messagep, const char *repository)
     if (config->RereadLogAfterVerify == LOGMSG_REREAD_ALWAYS ||
 	config->RereadLogAfterVerify == LOGMSG_REREAD_STAT)
     {
-	if(CVS_STAT (data.fname, &post_stbuf) != 0)
+	if(stat (data.fname, &post_stbuf) != 0)
 	    error (1, errno, "cannot find size of temp file %s", data.fname);
     }
 
@@ -633,6 +644,32 @@ logmsg_list_to_args_proc (Node *p, void *closure)
 	    case 's':
 		arg = p->key;
 		break;
+            case 'a':
+                li = p->data;
+                switch (li->type)
+                {
+                    case T_ADDED:
+                        arg = "added";
+                        break;
+                    case T_MODIFIED:
+                    case T_CONFLICT:
+                        arg = "modified";
+                        break;
+                    case T_REMOVED:
+                        arg = "removed";
+                        break;
+                    case T_TITLE:
+                        arg = "imported";
+                        break;
+		    default:
+			error (1, 0, "Unexpected action type %d.", li->type);
+                        break;
+                }
+                break;
+	    case 'T':
+		li = p->data;
+		arg = li->tag ? li->tag : "";
+		break;
 	    case 'V':
 		li = p->data;
 		arg = li->rev_old ? li->rev_old : "NONE";
@@ -753,6 +790,7 @@ logfile_write (const char *repository, const char *filter, const char *message,
        `}' as separators.  The format characters are:
 
          s = file name
+         a = file action (status)
 	 V = old version number (pre-checkin)
 	 v = new version number (post-checkin)
 
@@ -785,7 +823,8 @@ logfile_write (const char *repository, const char *filter, const char *message,
     /* %c = cvs_cmd_name
      * %p = shortrepos
      * %r = repository
-     * %{sVv} = file name, old revision (precommit), new revision (postcommit)
+     * %{saVv} = file name, file action (status)
+                 old revision (precommit), new revision (postcommit)
      */
     /*
      * Cast any NULL arguments as appropriate pointers as this is an
@@ -803,7 +842,7 @@ logfile_write (const char *repository, const char *filter, const char *message,
 #endif /* SERVER_SUPPORT */
 	                      "p", "s", srepos,
 	                      "r", "s", current_parsed_root->directory,
-	                      "sVv", ",", changes,
+	                      "saTVv", ",", changes,
 			      logmsg_list_to_args_proc, (void *) NULL,
 	                      (char *) NULL);
     if (!cmdline || !strlen (cmdline))
@@ -898,7 +937,7 @@ verifymsg_proc (const char *repository, const char *script, void *closure)
 	if (config->RereadLogAfterVerify == LOGMSG_REREAD_STAT)
 	{
 	    /* Remember the status of the temp file for later */
-	    if (CVS_STAT (vpd->fname, &(vpd->pre_stbuf)) != 0)
+	    if (stat (vpd->fname, &(vpd->pre_stbuf)) != 0)
 		error (1, errno, "cannot stat temp file %s", vpd->fname);
 
 	    /*
@@ -928,6 +967,8 @@ verifymsg_proc (const char *repository, const char *script, void *closure)
                                        "r", "s",
                                        current_parsed_root->directory,
                                        "l", "s", vpd->fname,
+				       "sV", ",", vpd->changes,
+				       logmsg_list_to_args_proc, (void *) NULL,
 				       (char *) NULL);
 
 #ifdef SUPPORT_OLD_INFO_FMT_STRINGS
