@@ -1,8 +1,9 @@
 /*
+ * Copyright (C) 2006 The Free Software Foundation, Inc.
+ * Copyright (c) 2004, Derek R. Price & Ximbiot <http://ximbiot.com>
+ * Copyright (c) 2001, Tony Hoyle
  * Copyright (c) 1992, Brian Berliner and Jeff Polk
  * Copyright (c) 1989-1992, Brian Berliner
- * Copyright (c) 2001, Tony Hoyle
- * Copyright (c) 2004, Derek R. Price & Ximbiot <http://ximbiot.com>
  *
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -10,9 +11,22 @@
  * Query CVS/Entries from server
  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* ANSI C headers.  */
 #include <stdbool.h>
 
+/* CVS headers.  */
+#include "ignore.h"
+#include "recurse.h"
+#include "wrapper.h"
+
+#include "cvs.h"
+
+
+
 static int ls_proc (int argc, char **argv, char *xwhere, char *mwhere,
                     char *mfile, int shorten, int local, char *mname,
                     char *msg);
@@ -100,9 +114,10 @@ ls (int argc, char **argv)
 		long_format = true;
 		break;
 	    case 'r':
-		show_tag = optarg;
+		parse_tagdate (&show_tag, &show_date, optarg);
 		break;
 	    case 'D':
+		if (show_date) free (show_date);
 		show_date = Make_Date (optarg);
 		break;
 	    case 'P':
