@@ -1,15 +1,31 @@
 /*
- * Copyright (c) 1992, Brian Berliner and Jeff Polk
- * Copyright (c) 1989-1992, Brian Berliner
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
+ *
+ * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
+ *                                  and others.
+ *
+ * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
+ * Portions Copyright (C) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS kit.  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* GNULIB headers.  */
 #include "getline.h"
-#include "history.h"
 #include "save-cwd.h"
 
+/* CVS headers.  */
+#include "history.h"
+#include "ignore.h"
+
+#include "cvs.h"
+
+
+
 #ifndef DBLKSIZ
 #define	DBLKSIZ	4096			/* since GNU ndbm doesn't define it */
 #endif
@@ -48,11 +64,11 @@ struct admin_file {
 };
 
 static const char *const loginfo_contents[] = {
-    "# The \"loginfo\" file controls where \"cvs commit\" log information\n",
-    "# is sent.  The first entry on a line is a regular expression which must match\n",
-    "# the directory that the change is being made to, relative to the\n",
-    "# $CVSROOT.  If a match is found, then the remainder of the line is a filter\n",
-    "# program that should expect log information on its standard input.\n",
+    "# The \"loginfo\" file controls where \"cvs commit\" log information is\n",
+    "# sent. The first entry on a line is a regular expression which must\n",
+    "# match the directory that the change is being made to, relative to the\n",
+    "# $CVSROOT.  If a match is found, then the remainder of the line is a\n",
+    "# filter program that should expect log information on its standard input.\n",
     "#\n",
     "# If the repository name does not match any of the regular expressions in this\n",
     "# file, the \"DEFAULT\" line is used, if it is specified.\n",
@@ -60,20 +76,27 @@ static const char *const loginfo_contents[] = {
     "# If the name ALL appears as a regular expression it is always used\n",
     "# in addition to the first matching regex or DEFAULT.\n",
     "#\n",
-    "# If any format strings are present in the filter, they will be replaced as follows:\n",
+    "# If any format strings are present in the filter, they will be replaced\n",
+    "# as follows:\n",
+    "#    %c = canonical name of the command being executed\n",
+#ifdef PROXY_SUPPORT
+    "#    %R = the name of the referrer, if any, otherwise the value NONE\n",
+#endif
     "#    %p = path relative to repository\n",
     "#    %r = repository (path portion of $CVSROOT)\n",
     "#    %{sVv} = attribute list = file name, old version number (pre-checkin),\n",
-    "#           new version number (post-checkin).  When either old or new revision is\n",
-    "#           unknown, doesn't exist, or isn't applicable, the string \"NONE\" will be\n",
-    "#           placed on the command line instead.\n",
+    "#           new version number (post-checkin).  When either old or new revision\n",
+    "#           is unknown, doesn't exist, or isn't applicable, the string \"NONE\"\n",
+    "#           will be placed on the command line instead.\n",
     "#\n",
-    "# Note that %{sVv} is a list operator and not all elements are necessary.  Thus %{sv} is\n",
-    "# a legal format string, but will only be replaced with file name and new revision.\n",
-    "# it also generates multiple arguments for each file being operated upon.  i.e. if two\n",
-    "# files, file1 & file2, are being commited from 1.1 to version 1.1.2.1 and from 1.1.2.2\n",
-    "# to 1.1.2.3, respectively, %{sVv} will generate the following six arguments in this\n",
-    "# order: file1, 1.1, 1.1.2.1, file2, 1.1.2.2, 1.1.2.3.\n",
+    "# Note that %{sVv} is a list operator and not all elements are necessary.\n",
+    "# Thus %{sv} is a legal format string, but will only be replaced with\n",
+    "# file name and new revision.\n",
+    "# It also generates multiple arguments for each file being operated upon.\n",
+    "# That is, if two files, file1 & file2, are being commited from 1.1 to\n",
+    "# version 1.1.2.1 and from 1.1.2.2 to 1.1.2.3, respectively, %{sVv} will\n",
+    "# generate the following six arguments in this order:\n",
+    "# file1, 1.1, 1.1.2.1, file2, 1.1.2.2, 1.1.2.3.\n",
     "#\n",
     "# For example:\n",
     "#DEFAULT (echo \"\"; id; echo %s; date; cat) >> $CVSROOT/CVSROOT/commitlog\n",
@@ -118,6 +141,10 @@ static const char *const verifymsg_contents[] = {
     "# If any of the above test failed, then the commit would be aborted.\n",
     "#\n",
     "# Format strings present in the filter will be replaced as follows:\n",
+    "#    %c = canonical name of the command being executed\n",
+#ifdef PROXY_SUPPORT
+    "#    %R = the name of the referrer, if any, otherwise the value NONE\n",
+#endif
     "#    %p = path relative to repository\n",
     "#    %r = repository (path portion of $CVSROOT)\n",
     "#    %l = name of log file to be verified.\n",
@@ -146,6 +173,10 @@ static const char *const commitinfo_contents[] = {
     "# of the line is the name of the filter to run.\n",
     "#\n",
     "# Format strings present in the filter will be replaced as follows:\n",
+    "#    %c = canonical name of the command being executed\n",
+#ifdef PROXY_SUPPORT
+    "#    %R = the name of the referrer, if any, otherwise the value NONE\n",
+#endif
     "#    %p = path relative to repository\n",
     "#    %r = repository (path portion of $CVSROOT)\n",
     "#    %{s} = file name, file name, ...\n",
@@ -164,7 +195,8 @@ static const char *const commitinfo_contents[] = {
 
 static const char *const taginfo_contents[] = {
     "# The \"taginfo\" file is used to control pre-tag checks.\n",
-    "# The filter on the right is invoked with the following arguments if no format strings are present:\n",
+    "# The filter on the right is invoked with the following arguments\n",
+    "# if no format strings are present:\n",
     "#\n",
     "# $1 -- tagname\n",
     "# $2 -- operation \"add\" for tag, \"mov\" for tag -F, and \"del\" for tag -d\n",
@@ -172,9 +204,15 @@ static const char *const taginfo_contents[] = {
     "# $4 -- repository\n",
     "# $5->  file revision [file revision ...]\n",
     "#\n",
-    "# If any format strings are present in the filter, they will be replaced as follows:\n",
-    "#    %b = branch mode = \"?\" (delete ops - unknown) | \"T\" (branch) | \"N\" (not branch)\n",
+    "# If any format strings are present in the filter, they will be replaced\n",
+    "# as follows:\n",
+    "#    %b = branch mode = \"?\" (delete ops - unknown) | \"T\" (branch)\n",
+    "#                     | \"N\" (not branch)\n",
     "#    %o = operation = \"add\" | \"mov\" | \"del\"\n",
+    "#    %c = canonical name of the command being executed\n",
+#ifdef PROXY_SUPPORT
+    "#    %R = the name of the referrer, if any, otherwise the value NONE\n",
+#endif
     "#    %p = path relative to repository\n",
     "#    %r = repository (path portion of $CVSROOT)\n",
     "#    %t = tagname\n",
@@ -189,8 +227,8 @@ static const char *const taginfo_contents[] = {
     "# name and old revision. it also generates multiple arguments for each file\n",
     "# being operated upon.  i.e. if two files, file1 & file2, are having a tag\n",
     "# moved from version 1.1 to version 1.1.2.9, %{sVv} will generate the\n",
-    "# following six arguments in this order: file1, 1.1, 1.1.2.9, file2, 1.1,\n",
-    "# 1.1.2.9.\n",
+    "# following six arguments in this order:\n",
+    "# file1, 1.1, 1.1.2.9, file2, 1.1, 1.1.2.9.\n",
     "#\n",
     "# A non-zero exit of the filter program will cause the tag to be aborted.\n",
     "#\n",
@@ -215,8 +253,12 @@ static const char *const preproxy_contents[] = {
     "# used to launch a dial up or VPN connection to the primary server's\n",
     "# network.\n",
     "#\n",
-    "# If any format strings are present in the filter, they will be replaced",
+    "# If any format strings are present in the filter, they will be replaced\n",
     "# as follows:\n",
+    "#    %c = canonical name of the command being executed\n",
+#ifdef PROXY_SUPPORT
+    "#    %R = the name of the referrer, if any, otherwise the value NONE\n",
+#endif
     "#    %p = path relative to repository (currently always \".\")\n",
     "#    %r = repository (path portion of $CVSROOT)\n",
     "#\n",
@@ -234,11 +276,15 @@ static const char *const preproxy_contents[] = {
 };
 
 static const char *const postadmin_contents[] = {
-    "# The \"postadmin\" file is called after the \"admin\" command finishes",
-    "# processing a directory.",
+    "# The \"postadmin\" file is called after the \"admin\" command finishes\n",
+    "# processing a directory.\n",
     "#\n",
-    "# If any format strings are present in the filter, they will be replaced",
+    "# If any format strings are present in the filter, they will be replaced\n",
     "# as follows:\n",
+    "#    %c = canonical name of the command being executed\n",
+#ifdef PROXY_SUPPORT
+    "#    %R = the name of the referrer, if any, otherwise the value NONE\n",
+#endif
     "#    %p = path relative to repository\n",
     "#    %r = repository (path portion of $CVSROOT)\n",
     "#\n",
@@ -261,8 +307,12 @@ static const char *const postproxy_contents[] = {
     "# This script might, for example, be used to shut down a dial up\n",
     "# or VPN connection to the primary server's network.\n",
     "#\n",
-    "# If any format strings are present in the filter, they will be replaced",
+    "# If any format strings are present in the filter, they will be replaced\n",
     "# as follows:\n",
+    "#    %c = canonical name of the command being executed\n",
+#ifdef PROXY_SUPPORT
+    "#    %R = the name of the referrer, if any, otherwise the value NONE\n",
+#endif
     "#    %p = path relative to repository (currently always \".\")\n",
     "#    %r = repository (path portion of $CVSROOT)\n",
     "#\n",
@@ -280,12 +330,18 @@ static const char *const postproxy_contents[] = {
 };
 
 static const char *const posttag_contents[] = {
-    "# The \"posttag\" file is called after the \"tag\" command finishes",
-    "# processing a directory.",
+    "# The \"posttag\" file is called after the \"tag\" command finishes\n",
+    "# processing a directory.\n",
     "#\n",
-    "# If any format strings are present in the filter, they will be replaced as follows:\n",
-    "#    %b = branch mode = \"?\" (delete ops - unknown) | \"T\" (branch) | \"N\" (not branch)\n",
+    "# If any format strings are present in the filter, they will be replaced\n",
+    "# as follows:\n",
+    "#    %b = branch mode = \"?\" (delete ops - unknown) | \"T\" (branch)\n",
+    "#                     | \"N\" (not branch)\n",
     "#    %o = operation = \"add\" | \"mov\" | \"del\"\n",
+    "#    %c = canonical name of the command being executed\n",
+#ifdef PROXY_SUPPORT
+    "#    %R = the name of the referrer, if any, otherwise the value NONE\n",
+#endif
     "#    %p = path relative to repository\n",
     "#    %r = repository (path portion of $CVSROOT)\n",
     "#    %t = tagname\n",
@@ -300,8 +356,8 @@ static const char *const posttag_contents[] = {
     "# name and old revision. it also generates multiple arguments for each file\n",
     "# being operated upon.  i.e. if two files, file1 & file2, are having a tag\n",
     "# moved from version 1.1 to version 1.1.2.9, %{sVv} will generate the\n",
-    "# following six arguments in this order: file1, 1.1, 1.1.2.9, file2, 1.1,\n",
-    "# 1.1.2.9.\n",
+    "# following six arguments in this order:\n",
+    "# file1, 1.1, 1.1.2.9, file2, 1.1, 1.1.2.9.\n",
     "#\n",
     "# The first entry on a line is a regular expression which is tested\n",
     "# against the directory that the change is being committed to, relative\n",
@@ -320,8 +376,12 @@ static const char *const postwatch_contents[] = {
     "# The \"postwatch\" file is called after any command finishes writing new\n",
     "# file attibute (watch/edit) information in a directory.\n",
     "#\n",
-    "# If any format strings are present in the filter, they will be replaced",
+    "# If any format strings are present in the filter, they will be replaced\n",
     "# as follows:\n",
+    "#    %c = canonical name of the command being executed\n",
+#ifdef PROXY_SUPPORT
+    "#    %R = the name of the referrer, if any, otherwise the value NONE\n",
+#endif
     "#    %p = path relative to repository\n",
     "#    %r = repository (path portion of $CVSROOT)\n",
     "#\n",
@@ -396,6 +456,10 @@ static const char *const notify_contents[] = {
     "# \"ALL\" or \"DEFAULT\" can be used in place of the regular expression.\n",
     "#\n",
     "# format strings are replaceed as follows:\n",
+    "#    %c = canonical name of the command being executed\n",
+#ifdef PROXY_SUPPORT
+    "#    %R = the name of the referrer, if any, otherwise the value NONE\n",
+#endif
     "#    %p = path relative to repository\n",
     "#    %r = repository (path portion of $CVSROOT)\n",
     "#    %s = user to notify\n",
@@ -412,7 +476,6 @@ static const char *const modules_contents[] = {
     "#	key [options] directory files...\n",
     "#\n",
     "# Where \"options\" are composed of:\n",
-    "#	-i prog		Run \"prog\" on \"cvs commit\" from top-level of module.\n",
     "#	-o prog		Run \"prog\" on \"cvs checkout\" of module.\n",
     "#	-e prog		Run \"prog\" on \"cvs export\" of module.\n",
     "#	-t prog		Run \"prog\" on \"cvs rtag\" of module.\n",
@@ -436,11 +499,16 @@ static const char *const modules_contents[] = {
 };
 
 static const char *const config_contents[] = {
-    "# Set this to \"no\" if pserver shouldn't check system users/passwords\n",
-    "#SystemAuth=no\n",
+    "# Set `SystemAuth' to `no' if pserver shouldn't check system users/passwords.\n",
+    "#SystemAuth=yes\n",
     "\n",
-    "# Put CVS lock files in this directory rather than directly in the repository.\n",
-    "#LockDir=/var/lock/cvs\n",
+    "# Set `LocalKeyword' to specify a local alias for a standard keyword.\n",
+    "#LocalKeyword=MYCVS=CVSHeader\n",
+    "\n",
+    "# Set `KeywordExpand' to `i' followed by a list of keywords to expand or\n",
+    "# `e' followed by a list of keywords to not expand.\n"
+    "#KeywordExpand=iMYCVS,Name,Date\n",
+    "#KeywordExpand=eCVSHeader\n",
     "\n",
 #ifdef PRESERVE_PERMISSIONS_SUPPORT
     "# Set `PreservePermissions' to `yes' to save file status information\n",
@@ -453,6 +521,9 @@ static const char *const config_contents[] = {
     "# command.\n",
     "#TopLevelAdmin=no\n",
     "\n",
+    "# Put CVS lock files in this directory rather than directly in the repository.\n",
+    "#LockDir=/var/lock/cvs\n",
+    "\n",
     "# Set `LogHistory' to `all' or `" ALL_HISTORY_REC_TYPES "' to log all transactions to the\n",
     "# history file, or a subset as needed (ie `TMAR' logs all write operations)\n",
     "#LogHistory=" ALL_HISTORY_REC_TYPES "\n",
@@ -473,7 +544,7 @@ static const char *const config_contents[] = {
     "#\n",
     "# The following string would enable all `cvs admin' commands for all\n",
     "# users:\n",
-    "#UserAdminOptions=aAbceIklLmnNostuU\n",
+    "#UserAdminOptions=aAbceIklLmnNostuU;execute;no-execute\n",
 #ifdef SUPPORT_OLD_INFO_FMT_STRINGS
     "\n",
     "# Set `UseNewInfoFmtStrings' to `no' if you must support a legacy system by\n",
@@ -486,6 +557,7 @@ static const char *const config_contents[] = {
     "# every `cvs import' command to behave as if the `-X' flag was\n",
     "# specified.\n",
     "#ImportNewFilesToVendorBranchOnly=no\n",
+#ifdef PROXY_SUPPORT
     "\n",
     "# Set `PrimaryServer' to the CVSROOT to the primary, or write, server when\n",
     "# establishing one or more read-only mirrors which serve as proxies for\n",
@@ -495,7 +567,6 @@ static const char *const config_contents[] = {
     "# For example:\n",
     "#\n",
     "#   PrimaryServer=:fork:localhost/cvsroot\n",
-#ifdef PROXY_SUPPORT
     "\n",
     "# Set `MaxProxyBufferSize' to the the maximum allowable secondary\n",
     "# buffer memory cache size before the buffer begins being stored to disk, in\n",
@@ -677,17 +748,14 @@ mkmodules (char *dir)
 	temp = make_tempfile ();
 	if (checkout_file (fileptr->filename, temp) == 0)
 	    rename_rcsfile (temp, fileptr->filename);
-#if 0
-	/*
-	 * If there was some problem other than the file not existing,
-	 * checkout_file already printed a real error message.  If the
-	 * file does not exist, it is harmless--it probably just means
-	 * that the repository was created with an old version of CVS
-	 * which didn't have so many files in CVSROOT.
+	/* else
+	 *   If there was some problem other than the file not existing,
+	 *   checkout_file already printed a real error message.  If the
+	 *   file does not exist, it is harmless--it probably just means
+	 *   that the repository was created with an old version of CVS
+	 *   which didn't have so many files in CVSROOT.
 	 */
-	else if (fileptr->errormsg)
-	    error (0, 0, fileptr->errormsg, fileptr->filename);
-#endif
+
 	if (unlink_file (temp) < 0
 	    && !existence_error (errno))
 	    error (0, errno, "cannot remove %s", temp);
@@ -808,15 +876,23 @@ checkout_file (char *file, char *temp)
     if (noexec)
 	return 0;
 
-    rcs = xmalloc (strlen (file) + 5);
-    strcpy (rcs, file);
-    strcat (rcs, RCSEXT);
+    rcs = Xasprintf ("%s%s", file, RCSEXT);
     if (!isfile (rcs))
     {
 	free (rcs);
 	return 1;
     }
+
     rcsnode = RCS_parsercsfile (rcs);
+    if (!rcsnode)
+    {
+	/* Probably not necessary (?); RCS_parsercsfile already printed a
+	   message.  */
+	error (0, 0, "Failed to parse `%s'.", rcs);
+	free (rcs);
+	return 1;
+    }
+
     retcode = RCS_checkout (rcsnode, NULL, NULL, NULL, NULL, temp, NULL, NULL);
     if (retcode != 0)
     {
@@ -844,7 +920,7 @@ write_dbmfile( char *temp )
     datum key, val;
     int len, cont, err = 0;
 
-    fp = open_file (temp, "r");
+    fp = xfopen (temp, "r");
     if ((db = dbm_open (temp, O_RDWR | O_CREAT | O_TRUNC, 0666)) == NULL)
 	error (1, errno, "cannot open dbm file %s for creation", temp);
     for (cont = 0; fgets (line, sizeof (line), fp) != NULL;)
@@ -1038,18 +1114,16 @@ rename_rcsfile (char *temp, char *real)
     char *rcs;
 
     /* Set "x" bits if set in original. */
-    rcs = xmalloc (strlen (real) + sizeof (RCSEXT) + 10);
-    (void) sprintf (rcs, "%s%s", real, RCSEXT);
+    rcs = Xasprintf ("%s%s", real, RCSEXT);
     statbuf.st_mode = 0; /* in case rcs file doesn't exist, but it should... */
-    if (CVS_STAT (rcs, &statbuf) < 0
+    if (stat (rcs, &statbuf) < 0
 	&& !existence_error (errno))
 	error (0, errno, "cannot stat %s", rcs);
     free (rcs);
 
     if (chmod (temp, 0444 | (statbuf.st_mode & 0111)) < 0)
 	error (0, errno, "warning: cannot chmod %s", temp);
-    bak = xmalloc (strlen (real) + sizeof (BAKPREFIX) + 10);
-    (void) sprintf (bak, "%s%s", BAKPREFIX, real);
+    bak = Xasprintf ("%s%s", BAKPREFIX, real);
 
     /* rm .#loginfo */
     if (unlink_file (bak) < 0
@@ -1111,8 +1185,7 @@ init (int argc, char **argv)
        which needs to be created.  */
     mkdir_if_needed (current_parsed_root->directory);
 
-    adm = xmalloc (strlen (current_parsed_root->directory) + sizeof (CVSROOTADM) + 2);
-    sprintf (adm, "%s/%s", current_parsed_root->directory, CVSROOTADM);
+    adm = Xasprintf ("%s/%s", current_parsed_root->directory, CVSROOTADM);
     mkdir_if_needed (adm);
 
     /* This is needed because we pass "fileptr->filename" not "info"
@@ -1151,7 +1224,7 @@ init (int argc, char **argv)
 		FILE *fp;
 		const char * const *p;
 
-		fp = open_file (info, "w");
+		fp = xfopen (info, "w");
 		for (p = fileptr->contents; *p != NULL; ++p)
 		    if (fputs (*p, fp) < 0)
 			error (1, errno, "cannot write %s", info);
@@ -1183,7 +1256,7 @@ init (int argc, char **argv)
     {
 	FILE *fp;
 
-	fp = open_file (info, "w");
+	fp = xfopen (info, "w");
 	if (fclose (fp) < 0)
 	    error (1, errno, "cannot close %s", info);
  
@@ -1201,7 +1274,7 @@ init (int argc, char **argv)
     {
 	FILE *fp;
 
-	fp = open_file (info, "w");
+	fp = xfopen (info, "w");
 	if (fclose (fp) < 0)
 	    error (1, errno, "cannot close %s", info);
  
