@@ -1,6 +1,12 @@
 /*
- * Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
+ * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
+ *                                  and others.
+ *
+ * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
+ * Portions Copyright (C) 1989-1992, Brian Berliner
+ * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
  *
@@ -8,10 +14,25 @@
  *
  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* Verify interface.  */
+#include "recurse.h"
+
+/* GNULIB headers.  */
 #include "save-cwd.h"
-#include "fileattr.h"
+
+/* CVS headers.  */
 #include "edit.h"
+#include "fileattr.h"
+#include "repos.h"
+#include "wrapper.h"
+
+#include "cvs.h"
+
+
 
 static int do_dir_proc (Node * p, void *closure);
 static int do_file_proc (Node * p, void *closure);
@@ -89,6 +110,9 @@ struct frame_and_entries {
  *     current directory (`.').
  *
  *   local
+ *     When true, operate on files in the local directory only (i.e., do not
+ *     recurse into subdirectories - this should generally correspond to the
+ *     user-supplied -R/-l options).
  *
  *   which
  *     specifies the kind of recursion.  There are several cases:
@@ -114,9 +138,22 @@ struct frame_and_entries {
  *       think those are the only two cases).
  *
  *   aflag
+ *     Whether any sitcky tags/dates/kopts should be reset.  Should correspond
+ *     to the -A command line option accepted by some CVS commands.
+ *
  *   locktype
+ *     CVS_LOCK_NONE	Do not create any locks.
+ *     CVS_LOCK_READ	Read lock files before handing to the processor
+ *     			functions.
+ *     CVS_LOCK_WRITE	Lock files for write before handing to the processor
+ *     			functions.  Deletes promotable locks when they exist.
+ *
  *   update_preload
+ *     The top level directory for printing in error messages (called
+ *     "update_dir" by many other functions).  If NULL, assume "".
+ *
  *   dosrcs
+ *     Whether to parse RCS files when possible.
  *
  *   repository_in
  *     keeps track of the repository string.  This is only for the remote mode,
@@ -272,7 +309,6 @@ start_recursion (FILEPROC fileproc, FILESDONEPROC filesdoneproc,
 		     * seems to be handled somewhere (else) but why should
 		     * it be a separate case?  Needs investigation...  */
 		    just_subdirs = 1;
-		free_cvsroot_t (root);
 	    }
 	}
 #endif
@@ -363,20 +399,26 @@ start_recursion (FILEPROC fileproc, FILESDONEPROC filesdoneproc,
 	     * string.
 	     */
 	    dir = xstrdup (argv[i]);
-	    /* It's okay to cast out last_component's const below since we know
-	     * we just allocated dir here and have control of it.
-	     */
-	    comp = (char *)last_component (dir);
+	    comp = last_component (dir);
 	    if (comp == dir)
 	    {
 		/* no dir component.  What we have is an implied "./" */
-		dir = xstrdup(".");
+		dir = xstrdup (".");
 	    }
-	    else
+	    else if (*comp)
 	    {
 		comp[-1] = '\0';
 		comp = xstrdup (comp);
 	    }
+	    else
+	    {
+		/* A root directory would have existed under UNIX and a missing
+		 * one specified on the command line (for instance,
+		 * `cvs up F:\' without drive F: mounted) is a fatal error
+		 * under Windows.
+		 */
+		error (1, ENOENT, "cannot process root directory `%s'", dir);
+	    }
 
 	    /* if this argument exists as a file in the current
 	       working directory tree, then add it to the files list.  */
@@ -384,9 +426,7 @@ start_recursion (FILEPROC fileproc, FILESDONEPROC filesdoneproc,
 	    if (!(which & W_LOCAL))
 	    {
 		/* If doing rtag, we've done a chdir to the repository. */
-		file_to_try = xmalloc( strlen( argv[i] )
-		                       + sizeof( RCSEXT ) + 1 );
-		sprintf (file_to_try, "%s%s", argv[i], RCSEXT);
+		file_to_try = Xasprintf ("%s%s", argv[i], RCSEXT);
 	    }
 	    else
 		file_to_try = xstrdup (argv[i]);
@@ -395,11 +435,8 @@ start_recursion (FILEPROC fileproc, FILESDONEPROC filesdoneproc,
 		addfile (&files_by_dir, dir, comp);
 	    else if (isdir (dir))
 	    {
-		if ((which & W_LOCAL) && isdir (CVSADM)
-#ifdef CLIENT_SUPPORT
-		    && !current_parsed_root->isremote
-#endif
-		    )
+		if ((which & W_LOCAL) && isdir (CVSADM) &&
+		    !current_parsed_root->isremote)
 		{
 		    /* otherwise, look for it in the repository. */
 		    char *tmp_update_dir;
@@ -418,10 +455,7 @@ start_recursion (FILEPROC fileproc, FILESDONEPROC filesdoneproc,
 
 		    /* look for it in the repository. */
 		    repos = Name_Repository (dir, tmp_update_dir);
-		    reposfile = xmalloc (strlen (repos)
-					 + strlen (comp)
-					 + 5);
-		    sprintf (reposfile, "%s/%s", repos, comp);
+		    reposfile = Xasprintf ("%s/%s", repos, comp);
 		    free (repos);
 
 		    if (!wrap_name_has (comp, WRAP_TOCVS) && isdir (reposfile))
@@ -678,22 +712,16 @@ do_recursion (struct recursion_frame *frame)
        directories, since we're guaranteed to have only one CVSROOT --
        our own.  */
 
-    if (
-	/* If -d was specified, it should override CVS/Root.
+    /* If -d was specified, it should override CVS/Root.
 
-	   In the single-repository case, it is long-standing CVS behavior
-	   and makes sense - the user might want another access method,
-	   another server (which mounts the same repository), &c.
+       In the single-repository case, it is long-standing CVS behavior
+       and makes sense - the user might want another access method,
+       another server (which mounts the same repository), &c.
 
-	   In the multiple-repository case, -d overrides all CVS/Root
-	   files.  That is the only plausible generalization I can
-	   think of.  */
-	CVSroot_cmdline == NULL
-
-#ifdef SERVER_SUPPORT
-	&& ! server_active
-#endif
-	)
+       In the multiple-repository case, -d overrides all CVS/Root
+       files.  That is the only plausible generalization I can
+       think of.  */
+    if (CVSroot_cmdline == NULL && !server_active)
     {
 	cvsroot_t *this_root = Name_Root (NULL, update_dir);
 	if (this_root != NULL)
@@ -703,7 +731,6 @@ do_recursion (struct recursion_frame *frame)
 		process_this_directory =
 		    !strcmp (original_parsed_root->original,
 			     this_root->original);
-		free_cvsroot_t (this_root);
 	    }
 	    else
 	    {
@@ -782,7 +809,8 @@ do_recursion (struct recursion_frame *frame)
 	    if (repository == NULL)
 	    {
 		Name_Repository (NULL, update_dir);
-	        assert (!"Not reached.  Please report this problem to <bug-cvs@gnu.org>");
+	        assert (!"Not reached.  Please report this problem to <"
+			PACKAGE_BUGREPORT ">");
 	    }
 	    /* find the files and fill in entries if appropriate */
 	    if (process_this_directory)
@@ -1060,10 +1088,7 @@ but CVS uses %s for its own purposes; skipping %s directory",
 	if (repository == NULL)
 	    newrepos = xstrdup ("");
 	else
-	{
-	    newrepos = xmalloc (strlen (repository) + strlen (dir) + 5);
-	    sprintf (newrepos, "%s/%s", repository, dir);
-	}
+	    newrepos = Xasprintf ("%s/%s", repository, dir);
     }
     else
     {
@@ -1137,22 +1162,16 @@ but CVS uses %s for its own purposes; skipping %s directory",
     /* Only process this directory if the root matches.  This nearly
        duplicates code in do_recursion. */
 
-    if (
-	/* If -d was specified, it should override CVS/Root.
+    /* If -d was specified, it should override CVS/Root.
 
-	   In the single-repository case, it is long-standing CVS behavior
-	   and makes sense - the user might want another access method,
-	   another server (which mounts the same repository), &c.
+       In the single-repository case, it is long-standing CVS behavior
+       and makes sense - the user might want another access method,
+       another server (which mounts the same repository), &c.
 
-	   In the multiple-repository case, -d overrides all CVS/Root
-	   files.  That is the only plausible generalization I can
-	   think of.  */
-	CVSroot_cmdline == NULL
-
-#ifdef SERVER_SUPPORT
-	&& ! server_active
-#endif
-	)
+       In the multiple-repository case, -d overrides all CVS/Root
+       files.  That is the only plausible generalization I can
+       think of.  */
+    if (CVSroot_cmdline == NULL && !server_active)
     {
 	cvsroot_t *this_root = Name_Root (dir, update_dir);
 	if (this_root != NULL)
@@ -1162,7 +1181,6 @@ but CVS uses %s for its own purposes; skipping %s directory",
 		process_this_directory =
 		    !strcmp (original_parsed_root->original,
 			     this_root->original);
-		free_cvsroot_t (this_root);
 	    }
 	    else
 	    {
@@ -1244,12 +1262,7 @@ but CVS uses %s for its own purposes; skipping %s directory",
 	    if (strcmp (dir, ".") == 0)
 		xframe.repository = xstrdup (repository);
 	    else
-	    {
-		xframe.repository = xmalloc (strlen (repository)
-					     + strlen (dir)
-					     + 2);
-		sprintf (xframe.repository, "%s/%s", repository, dir);
-	    }
+		xframe.repository = Xasprintf ("%s/%s", repository, dir);
 	}
 	else
 	    xframe.repository = NULL;
