@@ -1,6 +1,10 @@
 /*
- * Copyright (c) 1992, Mark D. Baushke
- * Copyright (c) 2002, Derek R. Price
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
+ *
+ * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
+ *                                  and others.
+ *
+ * Poritons Copyright (c) 1992, Mark D. Baushke
  *
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -10,17 +14,38 @@
  * Determine the path to the CVSROOT and set "Root" accordingly.
  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* Verify interface.  */
+#include "root.h"
+
+/* ANSI C headers.  */
+#include <assert.h>
+
+/* GNULIB headers.  */
 #include "getline.h"
 
+/* CVS headers.  */
+#include "repos.h"
+#include "stack.h"
+
+#include "cvs.h"
+
+
+
 /* Printable names for things in the current_parsed_root->method enum variable.
    Watch out if the enum is changed in cvs.h! */
 
 const char method_names[][16] = {
     "undefined", "local", "server (rsh)", "pserver",
-    "kserver", "gserver", "ext", "fork"
+    "kserver", "gserver", "ext", "extssh", "fork"
 };
 
+#define FQDN_CHARS \
+	"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUZWXYZ1234567890-."
+
 #ifndef DEBUG
 
 cvsroot_t *
@@ -36,6 +61,10 @@ Name_Root (const char *dir, const char *update_dir)
     char *cp;
     int len;
 
+    TRACE (TRACE_FLOW, "Name_Root (%s, %s)",
+	   dir ? dir : "(null)",
+	   update_dir ? update_dir : "(null)");
+
     if (update_dir && *update_dir)
 	xupdate_dir = update_dir;
     else
@@ -43,10 +72,8 @@ Name_Root (const char *dir, const char *update_dir)
 
     if (dir != NULL)
     {
-	cvsadm = xmalloc (strlen (dir) + sizeof (CVSADM) + 10);
-	(void) sprintf (cvsadm, "%s/%s", dir, CVSADM);
-	tmp = xmalloc (strlen (dir) + sizeof (CVSADM_ROOT) + 10);
-	(void) sprintf (tmp, "%s/%s", dir, CVSADM_ROOT);
+	cvsadm = Xasprintf ("%s/%s", dir, CVSADM);
+	tmp = Xasprintf ("%s/%s", dir, CVSADM_ROOT);
     }
     else
     {
@@ -72,7 +99,7 @@ Name_Root (const char *dir, const char *update_dir)
      * The assumption here is that the CVS Root is always contained in the
      * first line of the "Root" file.
      */
-    fpin = open_file (tmp, "r");
+    fpin = xfopen (tmp, "r");
 
     if ((len = getline (&root, &root_allocated, fpin)) < 0)
     {
@@ -105,17 +132,12 @@ Name_Root (const char *dir, const char *update_dir)
 	goto out;
     }
 
-    if (
-#ifdef CLIENT_SUPPORT
-        !ret->isremote &&
-#endif
-        !isdir (ret->directory))
+    if (!ret->isremote && !isdir (ret->directory))
     {
 	error (0, 0, "in directory %s:", xupdate_dir);
 	error (0, 0,
 	       "ignoring %s because it specifies a non-existent repository %s",
 	       CVSADM_ROOT, root);
-	free_cvsroot_t (ret);
 	ret = NULL;
 	goto out;
     }
@@ -150,14 +172,11 @@ Create_Root (const char *dir, const char *rootdir)
     if (rootdir != NULL)
     {
         if (dir != NULL)
-	{
-	    tmp = xmalloc (strlen (dir) + sizeof (CVSADM_ROOT) + 10);
-	    (void) sprintf (tmp, "%s/%s", dir, CVSADM_ROOT);
-	}
+	    tmp = Xasprintf ("%s/%s", dir, CVSADM_ROOT);
         else
 	    tmp = xstrdup (CVSADM_ROOT);
 
-        fout = open_file (tmp, "w+");
+        fout = xfopen (tmp, "w+");
         if (fprintf (fout, "%s\n", rootdir) < 0)
 	    error (1, errno, "write to %s failed", tmp);
         if (fclose (fout) == EOF)
@@ -185,11 +204,11 @@ Create_Root (const char *dir, const char *rootdir)
 const char *
 primary_root_translate (const char *root_in)
 {
+#ifdef PROXY_SUPPORT
     char *translated;
     static char *previous = NULL;
     static size_t len;
 
-#ifdef PROXY_SUPPORT
     /* This can happen, for instance, during `cvs init'.  */
     if (!config) return root_in;
 
@@ -231,11 +250,11 @@ primary_root_translate (const char *root_in)
 const char *
 primary_root_inverse_translate (const char *root_in)
 {
+#ifdef PROXY_SUPPORT
     char *translated;
     static char *previous = NULL;
     static size_t len;
 
-#ifdef PROXY_SUPPORT
     /* This can happen, for instance, during `cvs init'.  */
     if (!config) return root_in;
 
@@ -276,14 +295,14 @@ delconfig (Node *n)
 
 
 void
-root_allow_add (const char *arg)
+root_allow_add (const char *arg, const char *configPath)
 {
     Node *n;
 
     if (!root_allow) root_allow = getlist();
     n = getnode();
     n->key = xstrdup (arg);
-    n->data = parse_config (arg);
+    n->data = parse_config (arg, configPath);
     n->delproc = delconfig;
     addnode (root_allow, n);
 }
@@ -294,6 +313,12 @@ root_allow_free (void)
     dellist (&root_allow);
 }
 
+bool
+root_allow_used (void)
+{
+    return root_allow != NULL;
+}
+
 bool
 root_allow_ok (const char *arg)
 {
@@ -326,7 +351,7 @@ error 0 Server configuration missing --allow-root in inetd.conf\n");
  *   The config associated with ARG.
  */
 struct config *
-get_root_allow_config (const char *arg)
+get_root_allow_config (const char *arg, const char *configPath)
 {
     Node *n;
 
@@ -338,7 +363,7 @@ get_root_allow_config (const char *arg)
 	n = NULL;
 
     if (n) return n->data;
-    return parse_config (arg);
+    return parse_config (arg, configPath);
 }
 
 
@@ -381,16 +406,26 @@ new_cvsroot_t (void)
     newroot = xmalloc(sizeof(cvsroot_t));
 
     newroot->original = NULL;
+    newroot->directory = NULL;
     newroot->method = null_method;
-#ifdef CLIENT_SUPPORT
+    newroot->isremote = false;
+    newroot->sign = SIGN_DEFAULT;
+    newroot->sign_template = NULL;
+    newroot->openpgp_textmode = NULL;
+    newroot->sign_args = getlist ();
+    newroot->verify = VERIFY_DEFAULT;
+    newroot->verify_template = NULL;
+    newroot->verify_args = getlist ();
+#if defined CLIENT_SUPPORT || defined SERVER_SUPPORT
     newroot->username = NULL;
     newroot->password = NULL;
     newroot->hostname = NULL;
+    newroot->cvs_rsh = NULL;
+    newroot->cvs_server = NULL;
     newroot->port = 0;
-    newroot->directory = NULL;
     newroot->proxy_hostname = NULL;
     newroot->proxy_port = 0;
-    newroot->isremote = 0;
+    newroot->redirect = true;	/* Advertise Redirect support */
 #endif /* CLIENT_SUPPORT */
 
     return newroot;
@@ -398,14 +433,28 @@ new_cvsroot_t (void)
 
 
 
-/* Dispose of a cvsroot_t and its component parts */
-void
+/* Dispose of a cvsroot_t and its component parts.
+ *
+ * NOTE
+ *  It is dangerous for most code to call this function since parse_cvsroot
+ *  maintains a cache of parsed roots.
+ */
+static void
 free_cvsroot_t (cvsroot_t *root)
 {
+    assert (root);
     if (root->original != NULL)
 	free (root->original);
     if (root->directory != NULL)
 	free (root->directory);
+    if (root->sign_template)
+	free (root->sign_template);
+    if (root->openpgp_textmode)
+	free (root->openpgp_textmode);
+    dellist (&root->sign_args);
+    if (root->verify_template)
+	free (root->verify_template);
+    dellist (&root->verify_args);
 #ifdef CLIENT_SUPPORT
     if (root->username != NULL)
 	free (root->username);
@@ -417,6 +466,10 @@ free_cvsroot_t (cvsroot_t *root)
     }
     if (root->hostname != NULL)
 	free (root->hostname);
+    if (root->cvs_rsh != NULL)
+	free (root->cvs_rsh);
+    if (root->cvs_server != NULL)
+	free (root->cvs_server);
     if (root->proxy_hostname != NULL)
 	free (root->proxy_hostname);
 #endif /* CLIENT_SUPPORT */
@@ -439,12 +492,16 @@ free_cvsroot_t (cvsroot_t *root)
  *
  * RETURNS
  *	A pointer to a newly allocated cvsroot_t structure upon success and
- *	NULL upon failure.  The caller is responsible for disposing of
- *	new structures with a call to free_cvsroot_t().
+ *	NULL upon failure.  The caller should never dispose of this structure,
+ *	as it is stored in a cache, but the caller may rely on it not to
+ *	change.
  *
  * NOTES
  * 	This would have been a lot easier to write in Perl.
  *
+ *	Would it make sense to reimplement the root and config file parsing
+ *	gunk in Lex/Yacc?
+ *
  * SEE ALSO
  * 	free_cvsroot_t()
  */
@@ -454,14 +511,17 @@ parse_cvsroot (const char *root_in)
     cvsroot_t *newroot;			/* the new root to be returned */
     char *cvsroot_save;			/* what we allocated so we can dispose
 					 * it when finished */
+    char *cvsroot_copy, *p;		/* temporary pointers for parsing */
+#if defined(CLIENT_SUPPORT) || defined (SERVER_SUPPORT)
+    char *q;				/* temporary pointer for parsing */
     char *firstslash;			/* save where the path spec starts
 					 * while we parse
 					 * [[user][:password]@]host[:[port]]
 					 */
-    char *cvsroot_copy, *p, *q;		/* temporary pointers for parsing */
-#ifdef CLIENT_SUPPORT
     int check_hostname, no_port, no_password, no_proxy;
-#endif /* CLIENT_SUPPORT */
+#endif /* defined(CLIENT_SUPPORT) || defined (SERVER_SUPPORT) */
+    static List *cache = NULL;
+    Node *node;
 
     assert (root_in != NULL);
 
@@ -470,6 +530,11 @@ parse_cvsroot (const char *root_in)
      */
     TRACE (TRACE_FLOW, "parse_cvsroot (%s)", root_in);
 
+    if ((node = findnode (cache, root_in)))
+	return node->data;
+
+    assert (root_in);
+
     /* allocate some space */
     newroot = new_cvsroot_t();
 
@@ -500,28 +565,32 @@ parse_cvsroot (const char *root_in)
 	*p = '\0';
 	cvsroot_copy = ++p;
 
-#ifdef CLIENT_SUPPORT
+#if defined (CLIENT_SUPPORT) || defined (SERVER_SUPPORT)
 	/* Look for method options, for instance, proxy, proxyport.
 	 * Calling strtok again is saved until after parsing the method.
 	 */
 	method = strtok (method, ";");
-#endif /* CLIENT_SUPPORT */
+	if (!method)
+	    /* Could just exit now, but this keeps the error message in sync.
+	     */
+	    method = "";
+#endif /* defined (CLIENT_SUPPORT) || defined (SERVER_SUPPORT) */
 
 	/* Now we have an access method -- see if it's valid. */
 
-	if (strcmp (method, "local") == 0)
+	if (!strcasecmp (method, "local"))
 	    newroot->method = local_method;
-	else if (strcmp (method, "pserver") == 0)
+	else if (!strcasecmp (method, "pserver"))
 	    newroot->method = pserver_method;
-	else if (strcmp (method, "kserver") == 0)
+	else if (!strcasecmp (method, "kserver"))
 	    newroot->method = kserver_method;
-	else if (strcmp (method, "gserver") == 0)
+	else if (!strcasecmp (method, "gserver"))
 	    newroot->method = gserver_method;
-	else if (strcmp (method, "server") == 0)
+	else if (!strcasecmp (method, "server"))
 	    newroot->method = server_method;
-	else if (strcmp (method, "ext") == 0)
+	else if (!strcasecmp (method, "ext"))
 	    newroot->method = ext_method;
-	else if (strcmp (method, "fork") == 0)
+	else if (!strcasecmp (method, "fork"))
 	    newroot->method = fork_method;
 	else
 	{
@@ -529,12 +598,12 @@ parse_cvsroot (const char *root_in)
 	    goto error_exit;
 	}
 
-#ifdef CLIENT_SUPPORT
+#if defined (CLIENT_SUPPORT) || defined (SERVER_SUPPORT)
 	/* Parse the method options, for instance, proxy, proxyport */
 	while ((p = strtok (NULL, ";")))
 	{
 	    char *q = strchr (p, '=');
-	    if (q == NULL)
+	    if (!q && (strcasecmp (p, "sign") || strcasecmp (p, "no-sign")))
 	    {
 	        error (0, 0, "Option (`%s') has no argument in CVSROOT.",
                        p);
@@ -542,11 +611,12 @@ parse_cvsroot (const char *root_in)
 	    }
 
 	    *q++ = '\0';
-	    if (strcmp (p, "proxy") == 0)
+	    TRACE (TRACE_DATA, "CVSROOT option=`%s' value=`%s'", p, q);
+	    if (!strcasecmp (p, "proxy"))
 	    {
 		newroot->proxy_hostname = xstrdup (q);
 	    }
-	    else if (strcmp (p, "proxyport") == 0)
+	    else if (!strcasecmp (p, "proxyport"))
 	    {
 		char *r = q;
 		if (*r == '-') r++;
@@ -565,52 +635,143 @@ parse_cvsroot (const char *root_in)
 "CVSROOT may only specify a positive, non-zero, integer proxy port (not `%s').",
 			   q);
 	    }
+	    else if (!strcasecmp (p, "sign"))
+	    {
+		if (!q)
+		    newroot->sign = SIGN_ALWAYS;
+		else if (!strcasecmp (q, "auto") || !strcasecmp (q, "server"))
+		    newroot->sign = SIGN_DEFAULT;
+		else
+		{
+		    bool on;
+		    if (readBool ("CVSROOT", "sign", q, &on))
+		    {
+			if (on)
+			    newroot->sign = SIGN_ALWAYS;
+			else
+			    newroot->sign = SIGN_NEVER;
+		    }
+		    else
+			goto error_exit;
+		}
+	    }
+	    else if (!strcasecmp (p, "no-sign"))
+		newroot->sign = SIGN_NEVER;
+	    else if (!strcasecmp (p, "sign-template"))
+		newroot->sign_template = xstrdup (q);
+	    else if (!strcasecmp (p, "openpgp-textmode"))
+	    {
+		if (newroot->openpgp_textmode)
+		    free (newroot->openpgp_textmode);
+		newroot->openpgp_textmode = xstrdup (q);
+	    }
+	    else if (!strcasecmp (p, "no-openpgp-textmode"))
+	    {
+		if (newroot->openpgp_textmode)
+		    free (newroot->openpgp_textmode);
+		newroot->openpgp_textmode = xstrdup ("");
+	    }
+	    else if (!strcasecmp (p, "sign-arg"))
+		push_string (newroot->sign_args, q);
+	    else if (!strcasecmp (p, "no-verify"))
+		newroot->verify = VERIFY_OFF;
+	    else if (!strcasecmp (p, "verify"))
+	    {
+		if (!q)
+		    newroot->verify = VERIFY_FATAL;
+		else if (!strcasecmp (q, "fatal"))
+		    newroot->verify = VERIFY_FATAL;
+		else if (!strcasecmp (q, "warn"))
+		    newroot->verify = VERIFY_WARN;
+		else
+		{
+		    bool on;
+		    if (readBool ("CVSROOT", "verify", q, &on))
+		    {
+			if (on)
+			    newroot->verify = VERIFY_FATAL;
+			else
+			    newroot->verify = VERIFY_OFF;
+		    }
+		    else
+			goto error_exit;
+		}
+	    }
+	    else if (!strcasecmp (p, "verify-template"))
+		newroot->verify_template = xstrdup (q);
+	    else if (!strcasecmp (p, "verify-arg"))
+		push_string (newroot->verify_args, q);
+	    else if (!strcasecmp (p, "CVS_RSH"))
+	    {
+		/* override CVS_RSH environment variable */
+		if (newroot->method == ext_method)
+		    newroot->cvs_rsh = xstrdup (q);
+	    }
+	    else if (!strcasecmp (p, "CVS_SERVER"))
+	    {
+		/* override CVS_SERVER environment variable */
+		if (newroot->method == ext_method
+		    || newroot->method == fork_method)
+		    newroot->cvs_server = xstrdup (q);
+	    }
+	    else if (!strcasecmp (p, "Redirect"))
+		readBool ("CVSROOT", "Redirect", q, &newroot->redirect);
 	    else
 	    {
 	        error (0, 0, "Unknown option (`%s') in CVSROOT.", p);
 	        goto error_exit;
 	    }
 	}
-#endif /* CLIENT_SUPPORT */
+#endif /* defined (CLIENT_SUPPORT) || defined (SERVER_SUPPORT) */
     }
     else
     {
-	/* If the method isn't specified, assume EXT_METHOD if the string looks
-	   like a relative path and LOCAL_METHOD otherwise.  */
+	/* If the method isn't specified, assume LOCAL_METHOD unless the root
+	 * looks like [[user]@]server:/path.  Then assume EXT_METHOD.
+	 */
+	char *slash = strchr (cvsroot_copy, '/');
 
-	newroot->method = ((*cvsroot_copy != '/' && strchr (cvsroot_copy, '/'))
-			  ? ext_method
-			  : local_method);
+	if (slash)
+	{
+	    char *atchar;
+	    char *p;
+	    size_t len;
+
+	    *slash = '\0';
+	    atchar = strchr (cvsroot_copy, '@');
+	    p = (atchar) ? ++atchar : cvsroot_copy;
+	    len = strspn (p, FQDN_CHARS);
+
+	    /* FQDN_CHARS allows '.' and '-', but RFC 1035 forbids these as the
+	     * first character, and prohibits '-' as the last character. A server
+	     * name must be at least one character long.
+	     */
+	    if (len > 0 && p[0] != '.' && p[0] != '-' && p[len-1] != '-'
+		&& p[len] == ':')
+		newroot->method = ext_method;
+	    else
+		newroot->method = local_method;
+	    *slash = '/';
+	}
+	else
+	    newroot->method = local_method;
     }
 
     /*
      * There are a few sanity checks we can do now, only knowing the
      * method of this root.
      */
-#ifndef DEBUG
-    /* Why do we avoid these checks when DEBUG is set?  How is this used?  */
-# ifndef CLIENT_SUPPORT
-    if (newroot->method != local_method)
-    {
-	error (0, 0, "CVSROOT is set for a remote access method but your");
-	error (0, 0, "CVS executable doesn't support it.");
-	goto error_exit;
-    }
-# endif
-#endif /* ! DEBUG */
 
-#ifdef CLIENT_SUPPORT
     newroot->isremote = (newroot->method != local_method);
 
+#if defined (CLIENT_SUPPORT) || defined (SERVER_SUPPORT)
     if (readonlyfs && newroot->isremote)
 	error (1, 0,
 "Read-only repository feature unavailable with remote roots (cvsroot = %s)",
 	       cvsroot_copy);
 
     if ((newroot->method != local_method)
-#ifdef CLIENT_SUPPORT
 	&& (newroot->method != fork_method)
-#endif /* SERVER_SUPPORT */
        )
     {
 	/* split the string into [[user][:password]@]host[:[port]] & /path
@@ -700,7 +861,7 @@ parse_cvsroot (const char *root_in)
 	cvsroot_copy = firstslash;
 	*cvsroot_copy = '/';
     }
-#endif /* CLIENT_SUPPORT */
+#endif /* defined(CLIENT_SUPPORT) || defined (SERVER_SUPPORT) */
 
     /*
      * Parse the path for all methods.
@@ -716,52 +877,54 @@ parse_cvsroot (const char *root_in)
      * Do various sanity checks.
      */
 
-#ifdef CLIENT_SUPPORT
+#if defined(CLIENT_SUPPORT) || defined (SERVER_SUPPORT)
     if (newroot->username && ! newroot->hostname)
     {
 	error (0, 0, "Missing hostname in CVSROOT.");
 	goto error_exit;
     }
 
-    /* We won't have attempted to parse these without CLIENT_SUPPORT */
+    /* We won't have attempted to parse these without CLIENT_SUPPORT or
+     * SERVER_SUPPORT.
+     */
     check_hostname = 0;
     no_password = 1;
     no_proxy = 1;
     no_port = 0;
-#endif /* CLIENT_SUPPORT */
+#endif /* defined (CLIENT_SUPPORT) || defined (SERVER_SUPPORT) */
     switch (newroot->method)
     {
     case local_method:
-#ifdef CLIENT_SUPPORT
+#if defined(CLIENT_SUPPORT) || defined (SERVER_SUPPORT)
 	if (newroot->username || newroot->hostname)
 	{
 	    error (0, 0, "Can't specify hostname and username in CVSROOT");
 	    error (0, 0, "when using local access method.");
 	    goto error_exit;
 	}
-#endif /* CLIENT_SUPPORT */
+#endif /* defined(CLIENT_SUPPORT) || defined (SERVER_SUPPORT) */
 	/* cvs.texinfo has always told people that CVSROOT must be an
 	   absolute pathname.  Furthermore, attempts to use a relative
 	   pathname produced various errors (I couldn't get it to work),
 	   so there would seem to be little risk in making this a fatal
 	   error.  */
-	if (!isabsolute (newroot->directory))
+	if (!ISABSOLUTE (newroot->directory))
 	{
 	    error (0, 0, "CVSROOT must be an absolute pathname (not `%s')",
 		   newroot->directory);
 	    error (0, 0, "when using local access method.");
 	    goto error_exit;
 	}
-#ifdef CLIENT_SUPPORT
+#if defined(CLIENT_SUPPORT) || defined (SERVER_SUPPORT)
 	/* We don't need to check for these in :local: mode, really, since
 	 * we shouldn't be able to hit the code above which parses them, but
 	 * I'm leaving them here in lieu of assertions.
 	 */
 	no_port = 1;
 	/* no_password already set */
-#endif /* CLIENT_SUPPORT */
+#endif /* defined(CLIENT_SUPPORT) || defined (SERVER_SUPPORT) */
 	break;
-#ifdef CLIENT_SUPPORT
+#if defined(CLIENT_SUPPORT) || defined (SERVER_SUPPORT)
     case fork_method:
 	/* We want :fork: to behave the same as other remote access
            methods.  Therefore, don't check to see that the repository
@@ -773,7 +936,7 @@ parse_cvsroot (const char *root_in)
 	    goto error_exit;
 	}
 	newroot->hostname = xstrdup("server");  /* for error messages */
-	if (!isabsolute (newroot->directory))
+	if (!ISABSOLUTE (newroot->directory))
 	{
 	    error (0, 0, "CVSROOT must be an absolute pathname (not `%s')",
 		   newroot->directory);
@@ -784,26 +947,14 @@ parse_cvsroot (const char *root_in)
 	/* no_password already set */
 	break;
     case kserver_method:
-# ifndef HAVE_KERBEROS
-       	error (0, 0, "CVSROOT is set for a kerberos access method but your");
-	error (0, 0, "CVS executable doesn't support it.");
-	goto error_exit;
-# else
 	check_hostname = 1;
 	/* no_password already set */
 	break;
-# endif
     case gserver_method:
-# ifndef HAVE_GSSAPI
-	error (0, 0, "CVSROOT is set for a GSSAPI access method but your");
-	error (0, 0, "CVS executable doesn't support it.");
-	goto error_exit;
-# else
 	check_hostname = 1;
 	no_proxy = 0;
 	/* no_password already set */
 	break;
-# endif
     case server_method:
     case ext_method:
 	no_port = 1;
@@ -815,12 +966,12 @@ parse_cvsroot (const char *root_in)
 	no_proxy = 0;
 	check_hostname = 1;
 	break;
-#endif /* CLIENT_SUPPORT */
+#endif /* defined(CLIENT_SUPPORT) || defined (SERVER_SUPPORT) */
     default:
 	error (1, 0, "Invalid method found in parse_cvsroot");
     }
 
-#ifdef CLIENT_SUPPORT
+#if defined(CLIENT_SUPPORT) || defined (SERVER_SUPPORT)
     if (no_password && newroot->password)
     {
 	error (0, 0, "CVSROOT password specification is only valid for");
@@ -854,7 +1005,7 @@ parse_cvsroot (const char *root_in)
         error (0, 0, "and pserver connection methods.");
         goto error_exit;
     }
-#endif /*CLIENT_SUPPORT */
+#endif /* defined(CLIENT_SUPPORT) || defined (SERVER_SUPPORT) */
 
     if (*newroot->directory == '\0')
     {
@@ -864,6 +1015,12 @@ parse_cvsroot (const char *root_in)
     
     /* Hooray!  We finally parsed it! */
     free (cvsroot_save);
+
+    if (!cache) cache = getlist();
+    node = getnode();
+    node->key = xstrdup (newroot->original);
+    node->data = newroot;
+    addnode (cache, node);
     return newroot;
 
 error_exit:
@@ -884,6 +1041,9 @@ error_exit:
  * FIXME - we could cache the canonicalized version of a root inside the
  * cvsroot_t, but we'd have to un'const the input here and stop expecting the
  * caller to be responsible for our return value
+ *
+ * ASSUMPTIONS
+ *   ROOT->method == pserver_method
  */
 char *
 normalize_cvsroot (const cvsroot_t *root)
@@ -891,6 +1051,8 @@ normalize_cvsroot (const cvsroot_t *root)
     char *cvsroot_canonical;
     char *p, *hostname;
 
+    assert (root && root->hostname && root->directory);
+
     /* use a lower case hostname since we know hostnames are case insensitive */
     /* Some logic says we should be tacking our domain name on too if it isn't
      * there already, but for now this works.  Reverse->Forward lookups are
@@ -988,18 +1150,8 @@ local_cvsroot (const char *dir)
 char *program_name = "testing";
 char *cvs_cmd_name = "parse_cvsroot";		/* XXX is this used??? */
 
-/* Toy versions of various functions when debugging under unix.  Yes,
-   these make various bad assumptions, but they're pretty easy to
-   debug when something goes wrong.  */
-
-int
-isabsolute( const char *dir )
-{
-    return (dir && (*dir == '/'));
-}
-
 void
-main( int argc, char *argv[] )
+main (int argc, char *argv[])
 {
     program_name = argv[0];
 
@@ -1015,12 +1167,16 @@ main( int argc, char *argv[] )
 	exit (1);
     }
     printf ("CVSroot: %s\n", argv[1]);
-    printf ("current_parsed_root->method: %s\n", method_names[current_parsed_root->method]);
+    printf ("current_parsed_root->method: %s\n",
+	    method_names[current_parsed_root->method]);
     printf ("current_parsed_root->username: %s\n",
-	    current_parsed_root->username ? current_parsed_root->username : "NULL");
+	    current_parsed_root->username
+	      ? current_parsed_root->username : "NULL");
     printf ("current_parsed_root->hostname: %s\n",
-	    current_parsed_root->hostname ? current_parsed_root->hostname : "NULL");
-    printf ("current_parsed_root->directory: %s\n", current_parsed_root->directory);
+	    current_parsed_root->hostname
+	      ? current_parsed_root->hostname : "NULL");
+    printf ("current_parsed_root->directory: %s\n",
+	    current_parsed_root->directory);
 
    exit (0);
    /* NOTREACHED */
