@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
  * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
  *                                  and others.
@@ -13,6 +13,16 @@
 
 /* CVSroot data structures */
 
+#ifndef ROOT_H
+#define ROOT_H
+
+/* ANSI C includes.  */
+#include <stdbool.h>
+
+/* CVS Includes.  */
+#include "sign.h"
+#include "verify.h"
+
 /* Access method specified in CVSroot. */
 typedef enum {
     null_method = 0,
@@ -22,6 +32,7 @@ typedef enum {
     kserver_method,
     gserver_method,
     ext_method,
+    extssh_method,
     fork_method
 } CVSmethod;
 extern const char method_names[][16];	/* change this in root.c if you change
@@ -31,6 +42,23 @@ typedef struct cvsroot_s {
     char *original;		/* The complete source CVSroot string. */
     CVSmethod method;		/* One of the enum values above. */
     char *directory;		/* The directory name. */
+    bool isremote;		/* True if we are doing remote access. */
+    sign_state sign;		/* Whether to sign commits.  */
+    char *sign_template;	/* The template to use to launch the external
+				 * program to produce GPG signatures.
+				 */
+    List *sign_args;		/* Keep track of any additional arguments for
+				 * the sign tool.
+				 */
+    char *openpgp_textmode;     /* The arg GPG needs for text files.  */
+    verify_state verify;	/* Whether to verify checkouts.  */
+    char *verify_template;	/* The template to use to launch the external
+				 * program to verify GPG signatures.
+				 */
+    List *verify_args;		/* Keep track of any additional arguments for
+				 * the verify tool.
+				 */
+
 /* The following is required for servers now to allow Redirects to be sent
  * for remote roots when client support is disabled.
  */
@@ -47,7 +75,6 @@ typedef struct cvsroot_s {
 				 * used.
 				 */
     int proxy_port;		/* The port of the proxy or zero, as above. */
-    unsigned char isremote;	/* Nonzero if we are doing remote access. */
     bool redirect;		/* False if we are to disable redirects. */
 #endif /* defined (CLIENT_SUPPORT) || defined (SERVER_SUPPORT) */
 } cvsroot_t;
@@ -56,15 +83,17 @@ extern cvsroot_t *current_parsed_root;
 extern const cvsroot_t *original_parsed_root;
 
 cvsroot_t *Name_Root (const char *dir, const char *update_dir);
-void free_cvsroot_t (cvsroot_t *root_in);
 cvsroot_t *parse_cvsroot (const char *root)
 	__attribute__ ((__malloc__));
 cvsroot_t *local_cvsroot (const char *dir)
 	__attribute__ ((__malloc__));
 void Create_Root (const char *dir, const char *rootdir);
-void root_allow_add (const char *);
+void root_allow_add (const char *, const char *configPath);
 void root_allow_free (void);
+bool root_allow_used (void);
 bool root_allow_ok (const char *);
-struct config *get_root_allow_config (const char *arg);
+struct config *get_root_allow_config (const char *arg, const char *configPath);
 const char *primary_root_translate (const char *root_in);
 const char *primary_root_inverse_translate (const char *root_in);
+
+#endif /* ROOT_H */
