@@ -12,11 +12,18 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif /* CONFIG_H */
 
-#ifndef HAVE_UNISTD_H
-extern int execvp (char *file, char **argv);
-#endif
+/* Verify interface.  */
+#include "run.h"
+
+/* GNULIB headers.  */
+#include "wait.h"
+
+/* CVS headers.  */
+#include "cvs.h"
 
 
 
@@ -25,8 +32,8 @@ extern int execvp (char *file, char **argv);
  * arguments.  The argument to run_setup will be parsed into whitespace 
  * separated words and added to the global run_argv list.
  * 
- * Then, optionally call run_arg() for each additional argument that you'd like
- * to pass to the executed program.
+ * Then, optionally call run_add_arg() for each additional argument that you'd
+ * like to pass to the executed program.
  * 
  * Finally, call run_exec() to execute the program with the specified arguments.
  * The execvp() syscall will be used, so that the PATH is searched correctly.
@@ -34,13 +41,24 @@ extern int execvp (char *file, char **argv);
  */
 static char **run_argv;
 static int run_argc;
-static int run_argc_allocated;
+static size_t run_arg_allocated;
+
+
+
+void
+run_arg_free_p (int argc, char **argv)
+{
+    int i;
+    for (i = 0; i < argc; i++)
+	free (argv[i]);
+}
+
+
 
 /* VARARGS */
 void 
 run_setup (const char *prog)
 {
-    int i;
     char *run_prog;
     char *buf, *d, *s;
     size_t length;
@@ -49,14 +67,7 @@ run_setup (const char *prog)
     int dolastarg;
 
     /* clean out any malloc'ed values from run_argv */
-    for (i = 0; i < run_argc; i++)
-    {
-	if (run_argv[i])
-	{
-	    free (run_argv[i]);
-	    run_argv[i] = NULL;
-	}
-    }
+    run_arg_free_p (run_argc, run_argv);
     run_argc = 0;
 
     run_prog = xstrdup (prog);
@@ -111,19 +122,28 @@ run_setup (const char *prog)
 
 
 void
-run_add_arg (const char *s)
+run_add_arg_p (int *iargc, size_t *iarg_allocated, char ***iargv,
+	       const char *s)
 {
     /* allocate more argv entries if we've run out */
-    if (run_argc >= run_argc_allocated)
+    if (*iargc >= *iarg_allocated)
     {
-	run_argc_allocated += 50;
-	run_argv = xnrealloc (run_argv, run_argc_allocated, sizeof (char **));
+	*iarg_allocated += 50;
+	*iargv = xnrealloc (*iargv, *iarg_allocated, sizeof (char **));
     }
 
     if (s)
-	run_argv[run_argc++] = xstrdup (s);
+	(*iargv)[(*iargc)++] = xstrdup (s);
     else
-	run_argv[run_argc] = NULL;	/* not post-incremented on purpose! */
+	(*iargv)[*iargc] = NULL;	/* not post-incremented on purpose! */
+}
+
+
+
+void
+run_add_arg (const char *s)
+{
+    run_add_arg_p (&run_argc, &run_arg_allocated, &run_argv, s);
 }
 
 
@@ -431,8 +451,101 @@ run_popen (const char *cmd, const char *mode)
 
 
 
+/* Work around an OpenSSH problem: it can put its standard file
+   descriptors into nonblocking mode, which will mess us up if we
+   share file descriptions with it.  The simplest workaround is
+   to create an intervening process between OpenSSH and the
+   actual stderr.  */
+
+static void
+work_around_openssh_glitch (void)
+{
+    pid_t pid;
+    int stderr_pipe[2];
+    struct stat sb;
+
+    /* Do nothing unless stderr is a file that is affected by
+       nonblocking mode.  */
+    if (!(fstat (STDERR_FILENO, &sb) == 0
+          && (S_ISFIFO (sb.st_mode) || S_ISSOCK (sb.st_mode)
+              || S_ISCHR (sb.st_mode) || S_ISBLK (sb.st_mode))))
+       return;
+
+    if (pipe (stderr_pipe) < 0)
+       error (1, errno, "cannot create pipe");
+    pid = fork ();
+    if (pid < 0)
+       error (1, errno, "cannot fork");
+    if (pid != 0)
+    {
+       /* Still in child of original process.  Act like "cat -u".  */
+       char buf[1 << 13];
+       ssize_t inbytes;
+       pid_t w;
+       int status;
+
+       if (close (stderr_pipe[1]) < 0)
+           error (1, errno, "cannot close pipe");
+
+       while ((inbytes = read (stderr_pipe[0], buf, sizeof buf)) != 0)
+       {
+           size_t outbytes = 0;
+
+           if (inbytes < 0)
+           {
+               if (errno == EINTR)
+                   continue;
+               error (1, errno, "reading from pipe");
+           }
+
+           do
+           {
+               ssize_t w = write (STDERR_FILENO,
+                                  buf + outbytes, inbytes - outbytes);
+               if (w < 0)
+               {
+                   if (errno == EINTR)
+                     w = 0;
+                   if (w < 0)
+                     _exit (1);
+               }
+               outbytes += w;
+           }
+           while (inbytes != outbytes);
+       }
+
+       /* Done processing output from grandchild.  Propagate
+          its exit status back to the parent.  */
+       while ((w = waitpid (pid, &status, 0)) == -1 && errno == EINTR)
+           continue;
+       if (w < 0)
+           error (1, errno, "waiting for child");
+       if (!WIFEXITED (status))
+       {
+           if (WIFSIGNALED (status))
+               raise (WTERMSIG (status));
+           error (1, errno, "child did not exit cleanly");
+       }
+       _exit (WEXITSTATUS (status));
+    }
+
+    /* Grandchild of original process.  */
+    if (close (stderr_pipe[0]) < 0)
+       error (1, errno, "cannot close pipe");
+
+    if (stderr_pipe[1] != STDERR_FILENO)
+    {
+       if (dup2 (stderr_pipe[1], STDERR_FILENO) < 0)
+           error (1, errno, "cannot dup2 pipe");
+       if (close (stderr_pipe[1]) < 0)
+           error (1, errno, "cannot close pipe");
+    }
+}
+
+
+
 int
-piped_child (char *const *command, int *tofdp, int *fromfdp)
+piped_child (char *const *command, int *tofdp, int *fromfdp, bool fix_stderr)
 {
     int pid;
     int to_child_pipe[2];
@@ -450,28 +563,42 @@ piped_child (char *const *command, int *tofdp, int *fromfdp)
     setmode (from_child_pipe[1], O_BINARY);
 #endif
 
-#ifdef HAVE_VFORK
-    pid = vfork ();
-#else
     pid = fork ();
-#endif
     if (pid < 0)
 	error (1, errno, "cannot fork");
     if (pid == 0)
     {
-	if (dup2 (to_child_pipe[0], STDIN_FILENO) < 0)
-	    error (1, errno, "cannot dup2 pipe");
+	/* The child.  */
+
+	/* The parent uses these ends of the pipes.  */
 	if (close (to_child_pipe[1]) < 0)
 	    error (1, errno, "cannot close pipe");
 	if (close (from_child_pipe[0]) < 0)
 	    error (1, errno, "cannot close pipe");
+
+	/* Fool an unwitting child into reading from and writing to
+	 * the pipes.
+	 */
+	if (dup2 (to_child_pipe[0], STDIN_FILENO) < 0)
+	    error (1, errno, "cannot dup2 pipe");
 	if (dup2 (from_child_pipe[1], STDOUT_FILENO) < 0)
 	    error (1, errno, "cannot dup2 pipe");
 
-	/* Okay to cast out const below - execvp don't return anyhow.  */
+	/* Don't need these fds after the dup2.  */
+	if (close (to_child_pipe[0]) < 0)
+	    error (1, errno, "cannot close pipe");
+	if (close (from_child_pipe[1]) < 0)
+	    error (1, errno, "cannot close pipe");
+
+        if (fix_stderr)
+	    work_around_openssh_glitch ();
+
+	/* Okay to cast out const below - execvp don't return nohow.  */
 	execvp ((char *)command[0], (char **)command);
 	error (1, errno, "cannot exec %s", command[0]);
     }
+
+    /* The parent.  */
     if (close (to_child_pipe[0]) < 0)
 	error (1, errno, "cannot close pipe");
     if (close (from_child_pipe[1]) < 0)
@@ -488,7 +615,7 @@ int
 run_piped (int *tofdp, int *fromfdp)
 {
     run_add_arg (NULL);
-    return piped_child (run_argv, tofdp, fromfdp);
+    return piped_child (run_argv, tofdp, fromfdp, false);
 }
 
 
