@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
  * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
  *                                  and others.
@@ -13,20 +13,36 @@
  * Various useful functions for the CVS support code.
  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* Verify interface.  */
+#include "subr.h"
+
+/* ANSI headers.  */
+#ifdef HAVE_CVS_ADMIN_GROUP
+# include <grp.h>
+#endif
+
+/* GNULIB headers.  */
+#include "canonicalize.h"
+#include "canon-host.h"
 #include "getline.h"
 #include "vasprintf.h"
 #include "vasnprintf.h"
 
+/* CVS headers.  */
+#include "wrapper.h"
+
+#include "cvs.h"
+
 /* Get wint_t.  */
 #ifdef HAVE_WINT_T
 # include <wchar.h>
 #endif
 
-#if !defined HAVE_NANOSLEEP && !defined HAVE_USLEEP && defined HAVE_SELECT
-  /* use select as a workaround */
-# include "xselect.h"
-#endif /* !defined HAVE_NANOSLEEP && !defined HAVE_USLEEP && defined HAVE_SELECT */
+
 
 extern char *getlogin (void);
 
@@ -570,57 +586,6 @@ make_message_rcsvalid (const char *message)
 
 
 
-/*
- * file_has_conflict
- *
- * This function compares the timestamp of a file with ts_conflict set
- * to the timestamp on the actual file and returns TRUE or FALSE based
- * on the results.
- *
- * This function does not check for actual markers in the file and
- * file_has_markers() function should be called when that is interesting.
- *
- * ASSUMPTIONS
- *  The ts_conflict field is not NULL.
- *
- * RETURNS
- *  TRUE	ts_conflict matches the current timestamp.
- *  FALSE	The ts_conflict field does not match the file's
- *		timestamp.
- */
-int
-file_has_conflict (const struct file_info *finfo, const char *ts_conflict)
-{
-    char *filestamp;
-    int retcode;
-
-    /* If ts_conflict is NULL, there was no merge since the last
-     * commit and there can be no conflict.
-     */
-    assert (ts_conflict);
-
-    /*
-     * If the timestamp has changed and no
-     * conflict indicators are found, it isn't a
-     * conflict any more.
-     */
-
-#ifdef SERVER_SUPPORT
-    if (server_active)
-	retcode = ts_conflict[0] == '=' && ts_conflict[1] == '\0';
-    else 
-#endif /* SERVER_SUPPORT */
-    {
-	filestamp = time_stamp (finfo->file);
-	retcode = !strcmp (ts_conflict, filestamp);
-	free (filestamp);
-    }
-
-    return retcode;
-}
-
-
-
 /* Does the file FINFO contain conflict markers?  The whole concept
    of looking at the contents of the file to figure out whether there are
    unresolved conflicts is kind of bogus (people do want to manage files
@@ -660,92 +625,166 @@ out:
 
 
 
-/* Read the entire contents of the file NAME into *BUF.
-   If NAME is NULL, read from stdin.  *BUF
-   is a pointer returned from malloc (or NULL), pointing to *BUFSIZE
-   bytes of space.  The actual size is returned in *LEN.  On error,
-   give a fatal error.  The name of the file to use in error messages
-   (typically will include a directory if we have changed directory)
-   is FULLNAME.  MODE is "r" for text or "rb" for binary.  */
+bool
+file_contains_keyword (const struct file_info *finfo)
+{
+    FILE *fp;
+    bool result;
+    struct stat st;
+    char *content;
+
+    fp = CVS_FOPEN (finfo->file, "rb");
+    if (fp == NULL)
+	error (1, errno, "cannot open %s", finfo->fullname);
+    if (fstat (fileno (fp), &st))
+	error (1, errno, "cannot fstat `%s'", finfo->fullname);
+    content = xmalloc (st.st_size);
+    if (fread (content, sizeof *content, st.st_size, fp) < st.st_size)
+	error (1, errno, "Failed to read from `%s'", finfo->fullname);
+    result = contains_keyword (content, st.st_size);
+    if (fclose (fp) < 0)
+	error (0, errno, "cannot close %s", finfo->fullname);
+    free (content);
+    return result;
+}
+
+
+
+/* Read the entire contents of the file pointer IN into *BUF.  *BUF
+ * is a pointer returned from malloc (or NULL), pointing to *BUFSIZE
+ * bytes of space.  The actual size is returned in *LEN.  On error,
+ * give a fatal error.  The name of the file to use in error messages
+ * (typically will include a directory if we have changed directory)
+ * is FULLNAME.  MODE is "r" for text or "rb" for binary.
+ */
 void
-get_file (const char *name, const char *fullname, const char *mode, char **buf,
-	  size_t *bufsize, size_t *len)
+get_stream (FILE *in, const char *fullname, char **buf, size_t *bufsize,
+	    size_t *len)
 {
     struct stat s;
-    size_t nread;
-    char *tobuf;
-    FILE *e;
+    size_t off;
     size_t filesize;
 
-    if (name == NULL)
+    if (in == stdin)
     {
-	e = stdin;
+	fullname = "stdin";
 	filesize = 100;	/* force allocation of minimum buffer */
     }
     else
     {
 	/* Although it would be cleaner in some ways to just read
-	   until end of file, reallocating the buffer, this function
-	   does get called on files in the working directory which can
-	   be of arbitrary size, so I think we better do all that
-	   extra allocation.  */
-
-	if (CVS_STAT (name, &s) < 0)
-	    error (1, errno, "can't stat %s", fullname);
+	 * until end of file, reallocating the buffer, this function
+	 * does get called on files in the working directory which can
+	 * be of arbitrary size, so it makes some sense to just size
+	 * the buffer right the first time.
+	 */
+	if (fstat (fileno (in), &s) < 0)
+	    error (1, errno, "can't stat `%s'", fullname);
 
 	/* Convert from signed to unsigned.  */
 	filesize = s.st_size;
 
-	e = xfopen (name, mode);
+	/* Don't assume the file is positioned at the start.  */
+	rewind (in);
     }
 
-    if (*buf == NULL || *bufsize <= filesize)
+    if (!*buf || *bufsize < filesize + 1)
     {
+	*buf = xrealloc (*buf, filesize + 1);
 	*bufsize = filesize + 1;
-	*buf = xrealloc (*buf, *bufsize);
     }
 
-    tobuf = *buf;
-    nread = 0;
-    while (1)
+    off = 0;
+    while (true)
     {
 	size_t got;
 
-	got = fread (tobuf, 1, *bufsize - (tobuf - *buf), e);
-	if (ferror (e))
-	    error (1, errno, "can't read %s", fullname);
-	nread += got;
-	tobuf += got;
-
-	if (feof (e))
+	if (feof (in))
 	    break;
 
-	/* Allocate more space if needed.  */
-	if (tobuf == *buf + *bufsize)
-	{
-	    int c;
-	    long off;
+	got = fread (*buf + off, 1, *bufsize - off, in);
+	if (ferror (in))
+	    error (1, errno, "can't read `%s'", fullname);
+	off += got;
 
-	    c = getc (e);
-	    if (c == EOF)
-		break;
-	    off = tobuf - *buf;
-	    expand_string (buf, bufsize, *bufsize + 100);
-	    tobuf = *buf + off;
-	    *tobuf++ = c;
-	    ++nread;
-	}
+	/* Should only ever test positive reading from stdin.  */
+	if (off == *bufsize)
+	    /* Allocates *more* than *BUFSIZE and updates *BUFSIZE.  */
+	    *buf = x2realloc (*buf, bufsize);
     }
 
-    if (e != stdin && fclose (e) < 0)
-	error (0, errno, "cannot close %s", fullname);
+    *len = off;
 
-    *len = nread;
+    /* Force *BUF to be large enough to hold a null terminator.
+     * Should only ever test positive reading from stdin.
+     */
+    if (off == *bufsize)
+	/* Allocates *more* than *BUFSIZE and updates *BUFSIZE.  */
+	*buf = x2realloc (*buf, bufsize);
+    (*buf)[off] = '\0';
+}
 
-    /* Force *BUF to be large enough to hold a null terminator. */
-    if (nread == *bufsize)
-	expand_string (buf, bufsize, *bufsize + 1);
-    (*buf)[nread] = '\0';
+
+
+/* Read the entire contents of the file NAME into *BUF.  If NAME is NULL, read
+ * from stdin.  *BUF is a pointer returned from malloc (or NULL), pointing to
+ * *BUFSIZE bytes of space.  The actual size is returned in *LEN.  On error,
+ * give a fatal error.  The name of the file to use in error messages
+ * (typically will include a directory if we have changed directory)
+ * is FULLNAME.  MODE is "r" for text or "rb" for binary.
+ */
+void
+get_file (const char *name, const char *fullname, const char *mode, char **buf,
+	  size_t *bufsize, size_t *len)
+{
+    FILE *in;
+
+    if (name)
+	in = xfopen (name, mode);
+    else
+	in = stdin;
+
+    get_stream (in, fullname, buf, bufsize, len);
+    if (in != stdin && fclose (in) < 0)
+	error (0, errno, "cannot close `%s'", fullname);
+}
+
+
+
+/* Write DATA of length LEN to FILE, ignoring NOEXEC.  */
+void
+force_write_file (const char *file, const char *data, size_t len)
+{
+    FILE *fp;
+
+    TRACE (TRACE_FUNCTION,
+	   "force_write_file (%s, %s, %u)", file, data, (unsigned int)len);
+
+    if (!(fp = fopen (file, "wb")))
+	error (1, errno, "cannot create `%s'", file);
+
+    if (len > 0)
+    {
+	if (fwrite (data, sizeof *data, len, fp) != len)
+	    error (1, errno, "cannot write file `%s'", file);
+    }
+
+    if (fclose (fp) < 0)
+	error (1, errno, "cannot close `%s'", file);
+}
+
+
+
+/* Write DATA of length LEN to FILE, honoring NOEXEC.  */
+void
+write_file (const char *file, const char *data, size_t len)
+{
+    TRACE (TRACE_FUNCTION,
+	   "write_file (%s, %s, %u)", file, data, (unsigned int)len);
+
+    if (noexec) return;
+
+    force_write_file (file, data, len);
 }
 
 
@@ -772,7 +811,7 @@ resolve_symlink (char **filename)
 	   expedient hack seems to be looking at HAVE_READLINK.  */
 	char *newname = Xreadlink (*filename, rsize);
 	
-	if (isabsolute (newname))
+	if (ISABSOLUTE (newname))
 	{
 	    free (*filename);
 	    *filename = newname;
@@ -1740,15 +1779,23 @@ format_cmdline (const char *format, ...)
 #endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
 			    /* the *only* case possible without
 			     * SUPPORT_OLD_INFO_FORMAT_STRINGS
-			     * - !onearg */
-			    if (!inquotes)
+			     * - !onearg
+			     */
+			    /* Avoid adding an empty argument for NULL data.
+			     */
+			    if (!inquotes && b->data)
 			    {
 				doff = d - buf;
 				expand_string (&buf, &length, doff + 1);
 				d = buf + doff;
 				*d++ = '"';
 			    }
-			    outstr = cmdlineescape (inquotes ? inquotes : '"', b->data);
+			    if (b->data)
+				outstr = cmdlineescape (inquotes ? inquotes
+								 : '"',
+							b->data);
+			    else
+				outstr = xstrdup ("");
 #ifdef SUPPORT_OLD_INFO_FMT_STRINGS
 			} /* onearg */
 #endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
@@ -1762,7 +1809,7 @@ format_cmdline (const char *format, ...)
 			{
 			    free(outstr);
 #endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
-			    if (!inquotes)
+			    if (!inquotes && b->data)
 			    {
 				doff = d - buf;
 				expand_string (&buf, &length, doff + 1);
@@ -1832,17 +1879,6 @@ format_cmdline (const char *format, ...)
 
 
 
-/* Return true iff FILENAME is absolute.
-   Trivial under Unix, but more complicated under other systems.  */
-bool
-isabsolute (filename)
-    const char *filename;
-{
-    return ISABSOLUTE (filename);
-}
-
-
-
 /* Like xstrdup (), but can handle a NULL argument.
  */
 char *
@@ -1961,3 +1997,162 @@ xfopen (const char *name, const char *mode)
 	error (1, errno, "cannot open %s", name);
     return fp;
 }
+
+
+
+/* char *
+ * xcanonicalize_file_name (const char *path)
+ *
+ * Like canonicalize_file_name(), but exit on error.
+ *
+ * INPUTS
+ *  path	The original path.
+ *
+ * RETURNS
+ *  The path with any symbolic links, `.'s, or `..'s, expanded.
+ *
+ * ERRORS
+ *  This function exits with a fatal error if it fails to read the link for
+ *  any reason.
+ */
+char *
+xcanonicalize_file_name (const char *path)
+{
+    char *hardpath = canonicalize_file_name (path);
+    if (!hardpath)
+	error (1, errno, "Failed to resolve path: `%s'", path);
+    return hardpath;
+}
+
+
+
+/* Declared in main.c.  */
+extern char *server_hostname;
+
+/* Return true if OTHERHOST resolves to this host in the DNS.
+ *
+ * GLOBALS
+ *   server_hostname	The name of this host, as determined by the call to
+ *			xgethostname() in main().
+ *
+ * RETURNS
+ *   true	If OTHERHOST equals or resolves to HOSTNAME.
+ *   false	Otherwise.
+ */
+bool
+isThisHost (const char *otherhost)
+{
+    char *fqdno;
+    char *fqdns;
+    bool retval;
+
+    /* As an optimization, check the literal strings before looking up
+     * OTHERHOST in the DNS.
+     */
+    if (!strcasecmp (server_hostname, otherhost))
+	return true;
+
+    fqdno = canon_host (otherhost);
+    if (!fqdno)
+	error (1, 0, "Name lookup failed for `%s': %s",
+	       otherhost, ch_strerror ());
+    fqdns = canon_host (server_hostname);
+    if (!fqdns)
+	error (1, 0, "Name lookup failed for `%s': %s",
+	       server_hostname, ch_strerror ());
+
+    retval = !strcasecmp (fqdns, fqdno);
+
+    free (fqdno);
+    free (fqdns);
+    return retval;
+}
+
+
+
+/* Return true if two paths match, resolving symlinks.
+ */
+bool
+isSamePath (const char *path1_in, const char *path2_in)
+{
+    char *p1, *p2;
+    bool same;
+
+    if (!strcmp (path1_in, path2_in))
+	return true;
+
+    /* Path didn't match, but try to resolve any links that may be
+     * present.
+     */
+    if (!isdir (path1_in) || !isdir (path2_in))
+	/* To be resolvable, paths must exist on this server.  */
+	return false;
+
+    p1 = xcanonicalize_file_name (path1_in);
+    p2 = xcanonicalize_file_name (path2_in);
+    if (strcmp (p1, p2))
+	same = false;
+    else
+	same = true;
+
+    free (p1);
+    free (p2);
+    return same;
+}
+
+
+
+#ifdef HAVE_CVS_ADMIN_GROUP
+/* Return true if the current user should be allowed to run CVS admin commands,
+ * false otherwise.
+ */
+bool
+is_admin (void)
+{
+    struct group *grp;
+    struct group *getgrnam();
+
+    /* If the CVS_ADMIN_GROUP doesn't exist, assume that all users may run
+     * CVS admin commands.
+     */
+    if (!(grp = getgrnam (CVS_ADMIN_GROUP)))
+	return true;
+
+# ifdef HAVE_GETGROUPS
+    {
+	gid_t *grps;
+	int n, i;
+	size_t size;
+
+	/* get number of auxiliary groups */
+	n = getgroups (0, NULL);
+	if (n < 0)
+	    error (1, errno, "unable to get number of auxiliary groups");
+	size = xtimes (xsum (n, 1), sizeof *grps);
+	grps = size_in_bounds_p (size) ? xmalloc (size) : NULL;
+	if (!grps) error (1, EOVERFLOW, "too many groups");
+	n = getgroups (n, grps);
+	if (n < 0)
+	    error (1, errno, "unable to get list of auxiliary groups");
+	grps[n] = getgid();
+	for (i = 0; i <= n; i++)
+	    if (grps[i] == grp->gr_gid) break;
+	free (grps);
+	if (i > n)
+	    return false;
+    }
+# else /* !HAVE_GETGROUPS */
+    {
+	char *me = getcaller();
+	char **grnam;
+	
+	for (grnam = grp->gr_mem; *grnam; grnam++)
+	    if (strcmp (*grnam, me) == 0) break;
+	if (!*grnam && getgid() != grp->gr_gid)
+	    return false;
+    }
+# endif /* HAVE_GETGROUPS */
+
+    return true;
+}
+#endif /* HAVE_CVS_ADMIN_GROUP */
