@@ -18,6 +18,21 @@
 #ifndef SUBR_H
 # define SUBR_H
 
+/* FIXME - This shouldn't be needed here.  Any routine that needs to understand
+ * the underlying structure of an RCSNode should be in rcs*.c.
+ */
+#include "rcs.h"
+
+
+
+#ifdef USE_VMS_FILENAMES
+# define BAKPREFIX	"_$"
+#else /* USE_VMS_FILENAMES */
+# define BAKPREFIX	".#"		/* when rcsmerge'ing */
+#endif /* USE_VMS_FILENAMES */
+
+
+
 void expand_string (char **, size_t *, size_t);
 char *Xreadlink (const char *link, size_t size);
 void xrealloc_and_strcat (char **, size_t *, const char *);
@@ -33,10 +48,13 @@ char *previous_rev (RCSNode *rcs, const char *rev);
 char *gca (const char *rev1, const char *rev2);
 void check_numeric (const char *, int, char **);
 char *make_message_rcsvalid (const char *message);
-int file_has_conflict (const struct file_info *, const char *ts_conflict);
 int file_has_markers (const struct file_info *);
+bool file_contains_keyword (const struct file_info *finfo);
+void get_stream (FILE *, const char *, char **, size_t *, size_t *);
 void get_file (const char *, const char *, const char *,
                char **, size_t *, size_t *);
+void force_write_file (const char *file, const char *data, size_t len);
+void write_file (const char *file, const char *data, size_t len);
 void resolve_symlink (char **filename);
 char *backup_file (const char *file, const char *suffix);
 char *shell_escape (char *buf, const char *str);
@@ -72,8 +90,6 @@ char *format_cmdline (bool oldway, const char *srepos, const char *format, ...);
 char *format_cmdline (const char *format, ...);
 #endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
 
-bool isabsolute (const char *filename);
-
 /* Many, many CVS calls to xstrdup depend on it to return NULL when its
  * argument is NULL.
  */
@@ -89,4 +105,17 @@ bool readBool (const char *infopath, const char *option,
 	       const char *p, bool *val);
 
 FILE *xfopen (const char *, const char *);
+char *xcanonicalize_file_name (const char *path);
+bool isThisHost (const char *otherhost);
+bool isSamePath (const char *path1_in, const char *path2_in);
+
+# ifdef HAVE_CVS_ADMIN_GROUP
+bool is_admin (void);
+# else
+/* If the CVS_ADMIN_GROUP is not being used, then anyone may run admin
+ * commands.
+ */
+#   define is_admin()	true
+# endif
+
 #endif /* !SUBR_H */
