@@ -45,8 +45,10 @@ struct compress_buffer
 {
     /* The underlying buffer.  */
     struct buffer *buf;
+
     /* The compression information.  */
     z_stream zstr;
+    int level;
 };
 
 static void compress_error (int, int, z_stream *, const char *);
@@ -95,6 +97,7 @@ compress_buffer_initialize (struct buffer *buf, int input, int level,
     memset (n, 0, sizeof *n);
 
     n->buf = buf;
+    n->level = level;
 
     if (input)
 	zstatus = inflateInit (&n->zstr);
@@ -193,15 +196,23 @@ compress_buffer_input (void *closure, char *data, size_t need, size_t size,
 	bd->size = cb->zstr.avail_in;
 	bd->bufp = (char *) cb->zstr.next_in;
 
+	sofar = size - cb->zstr.avail_out;
+
 	if (zstatus == Z_STREAM_END)
+	{
+	    /* If we read any data, then return it, relying on the fact that
+	     * we will get Z_STREAM_END on the next read too.
+	     */
+	    if (sofar > 0) break;
+
+	    /* Otherwise, return EOF.  */
 	    return -1;
+	}
 
 	/* If we have obtained NEED bytes, then return, unless NEED is
            zero and we haven't obtained anything at all.  If NEED is
-           zero, we will keep reading from the underlying buffer until
-           we either can't read anything, or we have managed to
-           inflate at least one byte.  */
-	sofar = size - cb->zstr.avail_out;
+           zero, we will attempt at least one nonblocking read and see if
+	   we can inflate anything then.  */
 	if (sofar > 0 && sofar >= need)
 	    break;
 
@@ -209,19 +220,31 @@ compress_buffer_input (void *closure, char *data, size_t need, size_t size,
            point.  */
 	assert (bd->size == 0);
 
-	/* This will work well in the server, because this call will
-	   do an unblocked read and fetch all the available data.  In
-	   the client, this will read a single byte from the stdio
-	   stream, which will cause us to call inflate once per byte.
-	   It would be more efficient if we could make a call which
-	   would fetch all the available bytes, and at least one byte.  */
-
+	/* On the server, this will do an unblocking read of as much data as is
+	 * available.  On the client, with a blocking input descriptor and the
+	 * current fd_buffer implementation, this should read as much data as
+	 * is currently available, and at least 1 byte (or EOF), from the
+	 * underlying buffer.
+	 */
 	status = (*cb->buf->input) (cb->buf->closure, bd->text,
-				    need > 0 ? 1 : 0,
-				    BUFFER_DATA_SIZE, &nread);
-	if (status != 0)
+				    need ? 1 : 0, BUFFER_DATA_SIZE, &nread);
+
+	if (status == -2)
+	    /* Don't try to recover from memory allcoation errors.  */
 	    return status;
 
+	if (status != 0)
+	{
+	    /* If we read any data, then return it, relying on the fact that
+	     * we will get the same error reading the underlying buffer
+	     * on the next read too.
+	     */
+	    if (sofar > 0) break;
+
+	    /* Otherwise, return EOF.  */
+	    return status;
+	}
+
 	/* If we didn't read anything, then presumably the buffer is
            in nonblocking mode, and we should just get out now with
            whatever we've inflated.  */
@@ -242,19 +265,39 @@ compress_buffer_input (void *closure, char *data, size_t need, size_t size,
 
 
 
-/* Output data to a compression buffer.  */
+extern int gzip_level;
+
+/* Output data to a compression buffer.
+ *
+ * GLOBALS
+ *   gzip_level		If GZIP_LEVEL has changed to a value different from
+ *			CLOSURE->level, then set the compression level on the
+ *			stream to the new value.
+ */
 static int
 compress_buffer_output (void *closure, const char *data, size_t have,
 			size_t *wrote)
 {
     struct compress_buffer *cb = closure;
 
+    /* This is only used within the while loop below, but allocated here for
+     * efficiency.
+     */
+    static char *buffer = NULL;
+    if (!buffer)
+	buffer = xmalloc (BUFFER_DATA_SIZE);
+
+    if (cb->level != gzip_level)
+    {
+	cb->level = gzip_level;
+	deflateParams (&cb->zstr, gzip_level, Z_DEFAULT_STRATEGY);
+    }
+
     cb->zstr.avail_in = have;
     cb->zstr.next_in = (unsigned char *) data;
 
     while (cb->zstr.avail_in > 0)
     {
-	char buffer[BUFFER_DATA_SIZE];
 	int zstatus;
 
 	cb->zstr.avail_out = BUFFER_DATA_SIZE;
@@ -288,12 +331,18 @@ compress_buffer_flush (void *closure)
 {
     struct compress_buffer *cb = closure;
 
+    /* This is only used within the while loop below, but allocated here for
+     * efficiency.
+     */
+    static char *buffer = NULL;
+    if (!buffer)
+	buffer = xmalloc (BUFFER_DATA_SIZE);
+
     cb->zstr.avail_in = 0;
     cb->zstr.next_in = NULL;
 
     while (1)
     {
-	char buffer[BUFFER_DATA_SIZE];
 	int zstatus;
 
 	cb->zstr.avail_out = BUFFER_DATA_SIZE;
@@ -363,19 +412,10 @@ compress_buffer_shutdown_input (struct buffer *buf)
     struct compress_buffer *cb = buf->closure;
     int zstatus;
 
-    /* Pick up any trailing data, such as the checksum.  */
-    while (1)
-    {
-	int status;
-	size_t nread;
-	char buf[100];
-
-	status = compress_buffer_input (cb, buf, 0, sizeof buf, &nread);
-	if (status == -1)
-	    break;
-	if (status != 0)
-	    return status;
-    }
+    /* Don't make any attempt to pick up trailing data since we are shutting
+     * down.  If the client doesn't know we are shutting down, we might not
+     * see the EOF we are expecting.
+     */
 
     zstatus = inflateEnd (&cb->zstr);
     if (zstatus != Z_OK)
@@ -396,10 +436,15 @@ compress_buffer_shutdown_output (struct buffer *buf)
     struct compress_buffer *cb = buf->closure;
     int zstatus, status;
 
+    /* This is only used within the while loop below, but allocated here for
+     * efficiency.
+     */
+    static char *buffer = NULL;
+    if (!buffer)
+	buffer = xmalloc (BUFFER_DATA_SIZE);
+
     do
     {
-	char buffer[BUFFER_DATA_SIZE];
-
 	cb->zstr.avail_out = BUFFER_DATA_SIZE;
 	cb->zstr.next_out = (unsigned char *) buffer;
 
@@ -455,14 +500,18 @@ compress_buffer_shutdown_output (struct buffer *buf)
    it is an error we can't recover from.  */
 
 int
-gunzip_and_write (int fd, const char *fullname, unsigned char *buf,
-		  size_t size)
+gunzip_in_mem (const char *fullname, unsigned char *buf, size_t *size_in,
+	       char **retval)
 {
     size_t pos;
     z_stream zstr;
     int zstatus;
     unsigned char outbuf[32768];
     unsigned long crc;
+    size_t size = *size_in;
+
+    *retval = NULL;
+    *size_in = 0;
 
     if (size < 10)
     {
@@ -564,13 +613,13 @@ gunzip_and_write (int fd, const char *fullname, unsigned char *buf,
 	if (zstatus != Z_STREAM_END && zstatus != Z_OK)
 	{
 	    compress_error (0, zstatus, &zstr, fullname);
+	    if (*retval) free (*retval);
 	    return 1;
 	}
-	if (write (fd, outbuf, sizeof (outbuf) - zstr.avail_out) < 0)
-	{
-	    error (0, errno, "writing decompressed file %s", fullname);
-	    return 1;
-	}
+	*retval = xrealloc (*retval,
+			   *size_in + sizeof (outbuf) - zstr.avail_out);
+	memcpy (*retval + *size_in, outbuf, sizeof (outbuf) - zstr.avail_out);
+	*size_in += sizeof (outbuf) - zstr.avail_out;
 	crc = crc32 (crc, outbuf, sizeof (outbuf) - zstr.avail_out);
     } while (zstatus != Z_STREAM_END);
     zstatus = inflateEnd (&zstr);
@@ -584,6 +633,7 @@ gunzip_and_write (int fd, const char *fullname, unsigned char *buf,
     if (size - pos != 8)
     {
 	error (0, 0, "gzip data incomplete for %s (no trailer)", fullname);
+	if (*retval) free (*retval);
 	return 1;
     }
 
@@ -593,6 +643,7 @@ gunzip_and_write (int fd, const char *fullname, unsigned char *buf,
 		+ ((unsigned long)buf[pos + 3] << 24)))
     {
 	error (0, 0, "CRC error uncompressing %s", fullname);
+	if (*retval) free (*retval);
 	return 1;
     }
 
@@ -602,12 +653,36 @@ gunzip_and_write (int fd, const char *fullname, unsigned char *buf,
 			   + ((unsigned long)buf[pos + 7] << 24)))
     {
 	error (0, 0, "invalid length uncompressing %s", fullname);
+	if (*retval) free (*retval);
 	return 1;
     }
 
     return 0;
 }
 
+
+
+int
+gunzip_and_write (int fd, const char *fullname, unsigned char *buf,
+		  size_t size)
+{
+    char *dbuf;
+
+    if (gunzip_in_mem (fullname, buf, &size, &dbuf)) return 1;
+
+    if (write (fd, dbuf, size) < 0)
+    {
+	error (0, errno, "writing decompressed file %s", fullname);
+	free (dbuf);
+	return 1;
+    }
+
+    free (dbuf);
+    return 0;
+}
+
+
+
 /* Read all of FD and put the gzipped data (RFC1952/RFC1951) into *BUF,
    replacing previous contents of *BUF.  *BUF is xmalloc'd and *SIZE is
    its allocated size.  Put the actual number of bytes of data in
